--
-- Netreg database schema for PostgreSQL databases.
--
-- Parts (c) Kevin Grant, Griffith University, 2002.
-- Parts (c) Michael McMaster, Griffith University, 2002.
-- Parts (c) David Bussenschutt, Griffith University 2003.

drop table cookies;
create table cookies
(
    id          varchar(128)   not null,
    timeout     varchar(128)   not null,
    cookie_val  varchar(1024)  not null,

    CONSTRAINT PK_cookies PRIMARY KEY(id),
    CONSTRAINT CK_cookies UNIQUE(cookie_val)
); 

drop table campuses;
create table campuses
(
    campus	varchar(1024)	not null,
    server_ip	varchar(1024)	not null,

    CONSTRAINT PK_campuses PRIMARY KEY(campus,server_ip)
);

drop table subnets;
create table subnets
(
    cidr_subnet	varchar(15)	not null,
    cidr_mask   varchar(2)	not null,
    campus	varchar(1024)	not null,
    description varchar(1024),

    CONSTRAINT PK_subnets PRIMARY KEY(cidr_subnet, cidr_mask)
);

drop table subnet_domain;
create table subnet_domain
(
    cidr_subnet	varchar(1024) not null,
    cidr_mask	varchar(2)    not null,	
    domain      varchar(1024) not null,

    CONSTRAINT PK_subnet_domain PRIMARY KEY(cidr_subnet,cidr_mask,domain),
    CONSTRAINT FK_subnet_domain FOREIGN KEY(cidr_subnet,cidr_mask) REFERENCES subnets(cidr_subnet,cidr_mask)
        on delete cascade
);

drop table interface;
create table interface
(
    mac         varchar(128)   not null,
    hostname    varchar(1024)  not null,
    assetno     varchar(128)	not null,
    owner       varchar(1024)  not null,
    domain      varchar(1024)  not null,
    status      varchar(1)     constraint check_status_interface check(status in ('R','U','S')),
    statusdate  varchar(8)     not null,
    platform	varchar(1024),
    comments	varchar(1024),
    ip		varchar(15),
    cidr_subnet varchar(15),
    cidr_mask	varchar(2),
    group_id    varchar(1024),
    batch_id    numeric,



    CONSTRAINT PK_interface PRIMARY KEY(mac),
    CONSTRAINT FK_interface_static FOREIGN KEY(cidr_subnet,cidr_mask) REFERENCES subnets(cidr_subnet,cidr_mask),
    CONSTRAINT CK_interface UNIQUE (hostname,domain)
);

drop table ip_range;
create table ip_range
(
    min		varchar(15)	not null,
    max		varchar(15)	not null,
    cidr_subnet	varchar(15)	not null,
    cidr_mask	varchar(2)	not null,
    inclusive	varchar(1)	constraint check_ip_range_exclusive check(inclusive in('Y','N')),

    CONSTRAINT PK_ip_range PRIMARY KEY (min,max),
    CONSTRAINT FK_ip_range FOREIGN KEY (cidr_subnet,cidr_mask) REFERENCES subnets(cidr_subnet,cidr_mask),
    CONSTRAINT CK_ip_range_min UNIQUE(min),
    CONSTRAINT CK_ip_range_max UNIQUE(max)
);

drop table interface_log;
create table interface_log
(
    mac         varchar(128)   not null,
    hostname    varchar(1024)  not null,
    assetno     varchar(128)	not null,
    owner       varchar(1024)  not null,
    domain      varchar(1024)  not null,
    status      varchar(1)     constraint check_status_intreface_log check(status in ('R','U','S')),
    statusdate  varchar(8)	not null,
    entrydate   varchar(128)	not null,
    cso         varchar(1024)  not null,
    platform	varchar(1024),
    unregip     varchar(1024),
    comments    varchar(255)
);

drop table authentication;
create table authentication
(
    cidr_subnet	varchar(1024)	not null,
    cidr_mask	varchar(2)	not null,
    server	varchar(1024)	not null,
    method	varchar(128)	constraint check_auth_method check(method in ('NONE','IMAP','POP','FTP','LDAP')),

    CONSTRAINT PK_authentication PRIMARY KEY(cidr_subnet,cidr_mask,server,method),
    CONSTRAINT FK_authentication FOREIGN KEY(cidr_subnet,cidr_mask) REFERENCES subnets(cidr_subnet,cidr_mask)
        on delete cascade
);
