#----------------------------------------------------------------------------\
# NetReg 2.0 (pre-release devel version)
#       DHCPD Registration Frontend
#
# Version 1.3: 
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
# Version 2.0:
# Portions Copyright (C) 1999-2002 Peter Valian, Southwestern University.
# Portions Copyright (C) 2002 David Bussenschutt, Griffith University, Australia.
# Portions Copyright (C) 2002 Robert Lowe,  Lawrence University, Wisconsin.
# Portions Copyright (C) 2002-2003 Michael McMaster, Griffith University, Australia.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Version 1.3: http://www.netreg.org/
# Version 2.0: http://www.sourceforge.net/projects/netreg/
#
# Version 1.3 code contributions by:
#   Peter Valian <valianp@southwestern.edu>,  Southwestern University.
#     Author of Version 1.3rc2 
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#   Michael Grinnell <grinnell@american.edu>, American University
#     LDAP Authentication
#
# Version 2.0 code contributions by:
#   Robert Lowe <robert.h.lowe@lawrence.edu>, Lawrence University, Wisconsin.
#     CIDR (Subnet Masks etc) Handling/Notation
#     Alpha Testing, and bug chasing in David's Code.
#   David Bussenschutt <D.Bussenschutt@mailbox.gu.edu.au>, Griffith University, Queensland, Australia.
#     Much re-arrangement, and fixes, including but not limited to:
#     Perl 5 Syntax, and modules
#     File Locking
#     OMAPI Support
#     Remote dhcpdmanager daemon, and client support (functionally similar to omapi, but more fully implemented)
#     Improved Error Handling
#     Use of -w for improved code
#     LDAP group handling, for Permissions.
#     install_script
#   Michael McMaster <m4k3r@users.sourceforge.net>, Griffith University,  Queensland, Australia.
#     DBI/Oracle Support
#     HTML extraction/templating
#     Permissions handling (access controls)
#     Cookie based authentication
#     Interface for editing dhcp options/groups and hosts.
#     Synchronisation between hosts
#     Scalability improvements
#     Various code clean-ups
#     Consistency checker for host information 
#----------------------------------------------------------------------------/
package Authenticate;
use strict;
use warnings;
#----------------------------------------------------------------------------\\
#
# Exports
#
#----------------------------------------------------------------------------//
use Exporter();
our @ISA = qw(Exporter);

our @EXPORT = qw( );
our @EXPORT_OK = qw( auth_init user priv logon logoff is_valid_cookie );

#----------------------------------------------------------------------------\\
#
# Imported Modules
#
#----------------------------------------------------------------------------//
# If using mod_perl, the $FindBin::RealBin directory will be set ONCE only
# see startup.pl
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.

# Create a random hash to represent a logon session.
use MIME::Base64;
use FileHandle;

# Modules for authentication.
# Unnecessary modules may be commented out.
# Please note that LDAP currently requires that LDAPS is uncommented.
%%%vars{'AUTHFTP'}%%%use Net::FTP;
%%%vars{'AUTHPOP'}%%%use Mail::POP3Client;
%%%vars{'AUTHIMAP'}%%%use Net::IMAP::Simple;
%%%vars{'AUTHLDAP'}%%%use Net::LDAPS;

%%%vars{'AUTHLDAP'}%%%use Net::LDAP::Util qw(ldap_error_name ldap_error_text ldap_error_desc);
%%%vars{'AUTHLDAP'}%%%use Net::LDAP::Constant qw(LDAP_COMPARE_TRUE);

#Here we need to define LDAP_COMPARE_TRUE (and the Util subroutines) to be something/anything to prevent compiler errors when LDAP is not being used, so this Empty stub module does this for us.
%%%vars{'NOAUTHLDAP'}%%%use EmptyStubNetLDAP qw(ldap_error_name ldap_error_text ldap_error_desc); 
%%%vars{'NOAUTHLDAP'}%%%use constant LDAP_COMPARE_TRUE => 5;





# Netreg modules.
use NetregMisc qw( open_with_lock );
use Variables qw( $LDAP_BASE $LDAP_USERS $IP $AUTHFILE $COOKIE_TIMEOUT
                  $GLOBAL_AUTH_SVR $GLOBAL_AUTH_METHOD);
use Html qw(register_error admin_error set_cookie get_cookies );
use Subnets qw( what_is_my_registered_subnet );

#----------------------------------------------------------------------------\\
#
# Constants
#
#----------------------------------------------------------------------------//
# Optional mod_perl support.
# Sets a constant, and ensures the correct exit() function is called.
use constant IS_MODPERL => $ENV{MOD_PERL};
use subs qw(exit);
# Select the correct exit function
*exit = IS_MODPERL ? \&Apache::exit : sub { CORE::exit };

#----------------------------------------------------------------------------\\
#
# Global Variables
#
# These are shared within this perl module only.
# These variables are specifically shared between executions if using mod_perl
#----------------------------------------------------------------------------//
# A hash of usernames to an array ref of their privilege strings.
# This is only read on startup.
my %privileges;

# Details of the current user.
# WARNING: these will be meaningless (and dangerous) unless auth_init
# is called on each executable script.
my $current_user;

#----------------------------------------------------------------------------\\
#
# SQL init
#
#----------------------------------------------------------------------------//
# Prepare database queries. This function is only called by db_connect in
# Database.pm

my ($sql_get_auth_servers,$sql_delete_cookie,$sql_new_cookie,$sql_update_cookie,
    $sql_get_cookie);
sub db_init {
    my $dbh = shift;

    $sql_get_auth_servers = $dbh->prepare("select server, method from authentication where ".
          "cidr_subnet = ? and cidr_mask = ?");

    $sql_delete_cookie = $dbh->prepare("delete from cookies where id = ?");
    $sql_new_cookie = $dbh->prepare("insert into cookies values (?,?,?)");
    $sql_update_cookie = $dbh->prepare("update cookies set timeout = ? where cookie_val = ?");
    $sql_get_cookie = $dbh->prepare("select * from cookies where cookie_val = ?");
}

#----------------------------------------------------------------------------\\
#
# Automatically Executed Methods
#
#----------------------------------------------------------------------------//

# Read the auth.dat file to get the privilege levels of each user.
BEGIN {
    open (AUTH, "<$AUTHFILE") || die "Could not open auth file $AUTHFILE.\n$!\n";

    while (<AUTH>) {
        next if /^#/;
        next if /^\s+$/;
        s/\#.*$//;
        if ($_ !~ /^([a-z0-9_]+)\s+([a-z0-9_\,\.\/]+)\s*$/i) {
            die "$AUTHFILE - invalid format.\n";
        }
        my @array = split(/,/, $2);
        $privileges{$1} = \@array;
    }
    close AUTH;
}

#----------------------------------------------------------------------------\\
#
# Exported Methods
#
#----------------------------------------------------------------------------//

#----------------------------------------------------------------------------\\
# auth_init: initialise the authenticate module for use. Probably not necessary
# unless using mod_perl support.
#
# Returns: nothing useful.
#----------------------------------------------------------------------------//
sub auth_init {
    $current_user = "";
}

#----------------------------------------------------------------------------\\
# user: return the currently logged on user. Result is meaningless unless
# 'logon' or 'is_valid_cookie' has been called.
#
# Returns: nothing useful.
#----------------------------------------------------------------------------//
sub user {
    return $current_user;
}

#----------------------------------------------------------------------------\\
# priv: return the privilege level of the currently logged on user. 
# Result is meaningless unless 'logon' or 'is_valid_cookie' has been called.
#
# Parameters: A ACL string to check against.
#
# Returns: True if allowed
#----------------------------------------------------------------------------//
sub priv {
    my $acl = shift;

    foreach (@{$privileges{$current_user}}) {
        return 1 if ($_ eq 'super');
    }

    $acl =~ /group_(.+)/;
    my $group = $1 || '';
    # simply grep though the privileges to find the acl.
    foreach (@{$privileges{$current_user}}) {
        return 1 if ($_ eq $acl);
        return 1 if ($acl =~ /^group_.*$/ && $_ eq 'dhcp_all');
        return 1 if ($acl =~ /^option_/ && $_ eq 'option_all');
        s/^group_//;
        return 1 if ($group && $group =~ /$_/);
    }
    return undef;
}

#----------------------------------------------------------------------------\\
# logon: authenticates a user with the system, sends a cookie to the browser,
# and saves the cookie in $COOKIEJAR.
#
# Parameters: Username to log on
#             Password of the user
#
# Returns: '1' for a normal user, undef/0 on error.
#----------------------------------------------------------------------------//
sub logon {
    my $user = shift;
    my $pass = shift;

    my $priv = authenticate ($user, $pass);

    # Create an essentially random cookie.
    my $cookie_val;
    my $random_fh = new FileHandle "</dev/random";
    if ($random_fh) {
        my $bytes;
        sysread $random_fh, $bytes, 24; # 192 bits of random data.
        close $random_fh;
        $cookie_val = encode_base64($bytes);
    } else {
        $cookie_val = join('', map( chr(33+rand 93), 1 .. 64)) ;
        print STDERR "Warning: Using lower strength random numbers. Consider making /dev/random available\n";
    }

    chomp $cookie_val; # just in case there's a \n on it

    # send cookie to the web browser
    set_cookie("netreg-session", $cookie_val, undef);

    my $timeout = time() + $COOKIE_TIMEOUT;

    # Delete any previous entries
    # TODO transaction ??
    $sql_delete_cookie->execute($user);
    $sql_delete_cookie->finish();

    # save new cookie. 
    $sql_new_cookie->execute($user, $timeout, $cookie_val);
    $sql_new_cookie->finish();
    
    $current_user = $user;
    return 1;
}

#----------------------------------------------------------------------------\\
# logoff: removes a session from the COOKIEJAR.
#
# Parameters: Username to log off
#
# Returns: Nothing useful
#----------------------------------------------------------------------------//
sub logoff {
    my $user = shift;

    $sql_delete_cookie->execute($user);
    $sql_delete_cookie->finish();
}

#----------------------------------------------------------------------------\\
# is_valid_cookie: checks the given cookie against the list of known cookies,
# and updates the timeout value on the stored cookie.
#
# Parameters: cookie value.
#
# Returns: user
#----------------------------------------------------------------------------//
sub is_valid_cookie {
    my $cookie = shift;

    chomp $cookie; 

    $sql_get_cookie->execute($cookie); 
    my ($user, $timeout, $cookie_val) = $sql_get_cookie->fetchrow_array(); 
    $sql_get_cookie->finish();

    chomp $cookie_val if defined $cookie_val;

    if ($timeout && time() < $timeout) {
        # The cookie has been found. Update the timeout
        my $newTimeout = time() + $COOKIE_TIMEOUT;

        $sql_update_cookie->execute($newTimeout, $cookie_val);
        $sql_update_cookie->finish();
        $current_user = $user;
        return $user;
    }
    return undef;
}


#----------------------------------------------------------------------------\\
#
# Non-Exported Methods
#
#----------------------------------------------------------------------------//


#----------------------------------------------------------------------------\\
# authenticate: Use the authentication method specified for the users current
#  subnet (based on the users current IP stored in global variable $IP)
#
# Parameters: username, password.
#
# Returns: '1' on success.
#          Does not return on failure. 
#----------------------------------------------------------------------------//
sub authenticate {
    my $user = shift;
    my $pass = shift;
    
    if ($user eq "" || $pass eq "") { 
        register_error("Username or Password cannot be blank.");
    }

    # get a list of all authentication servers that are 
    # defined in the database for the subnet I'm currently on....
    my %servers = ();
    my $server_count = 0;
    my ($subnet, $mask) = Subnets::what_is_my_registered_subnet($IP);
    $subnet = '' unless defined $subnet;
    $mask = '' unless defined $mask;
    $sql_get_auth_servers->execute($subnet, $mask);
    while (my @row = $sql_get_auth_servers->fetchrow_array) {
        $server_count++;
        $servers{$row[0]} = $row[1];
    }
    $sql_get_auth_servers->finish();


    # if we are NOT in a subnet that is part of netreg, then use the
    # global authentication server as defined in Variables.pm
    if (!$server_count) {
      my $auth_perms = undef;

        for($GLOBAL_AUTH_METHOD) { # a switch statement
            study $_;
            /POP/i and do {$auth_perms = auth_pop($user,$pass,$GLOBAL_AUTH_SVR);last; };
            /FTP/i and do {$auth_perms = auth_ftp($user,$pass,$GLOBAL_AUTH_SVR);last; };
            /IMAP/i and do {$auth_perms = auth_imap($user,$pass,$GLOBAL_AUTH_SVR);last; };
            /LDAP/i and do {$auth_perms = auth_ldap($user,$pass,$GLOBAL_AUTH_SVR);last; };
            /NONE/i and do {return 1; last; };

            register_error("No Global Authentication Method defined ($GLOBAL_AUTH_METHOD not valid in Variables.pm)");
        }
        # OK, so we know our authorisation level/s etc, let's tell the user/return it! ...
        if ((!$auth_perms) || ($auth_perms < 0)) {
            register_error("Your IP is $IP  No Auth Server defined for subnet: $subnet / $mask. Tried Global Authentication Server($GLOBAL_AUTH_METHOD) -  currently un-available.");
            return 0;
        } else {
          # return the authorisation level of the user if they were checked OK
          # this may include a level of 0 if the auth_xxx was good, but the username and/or password was wrong.
          return $auth_perms;
        }

    # if we ARE in a subnet that's got an authentication server defined for it in the database
    # then we need to use the server and type as defined there to authenticate the user....
    } else {
      my $auth_perms = undef;

        # Try each auth server for the subnet.
        foreach my $key ( keys %servers ) {
            my $result = 0;
        
            for($servers{$key}) { # a switch statement
                study $_;
                /POP/i and do {$result = auth_pop($user,$pass,$key);last; };
                /FTP/i and do {$result = auth_ftp($user,$pass,$key);last; };
                /IMAP/i and do {$result = auth_imap($user,$pass,$key);last; };
                /LDAP/i and do {$result = auth_ldap($user,$pass,$key);last; };
                /NONE/i and do {return 1; last; };
                register_error("Subnet Authentication Method ($_) defined in database, but not supported!");
            }
            next if ($result < 0);  # Could not connect to the auth server, so try the next.
	    
            # remember authorisation level (0=denied, >=1 means let in!)
            $auth_perms = $result;
            last;
        }

        # OK, so we know our authorisation level/s etc, let's tell the user/return it! ...
        if ((!$auth_perms) || ($auth_perms < 0)) {
            register_error("Your IP is: $IP  Auth server/s for subnet ($subnet / $mask)  currently un-available.");
            return 0;
        } else {
          # return the authorisation level of the user if they were checked OK
          # this may include a level of 0 if the auth_xxx was good, but the username and/or password was wrong.
          return $auth_perms;
        }
    }
    # we never get here.
}

#----------------------------------------------------------------------------\\
# auth_pop: Authenticate against a POP3 server.
#
# Parameters: username, password, servername.
#
# Returns: -1 if no connection can be made to the server
#          privilege level if successful
#          Does not return if username/password invalid.
#----------------------------------------------------------------------------//
sub auth_pop {
    my ($user, $pass, $auth_svr) = @_;
    
    my $pop=new Mail::POP3Client($user,$pass,$auth_svr);
    if (!defined $pop || $pop eq '') {
        return -1;  # No handle; try next authentication server/method pair
    }
    my $alive=$pop->POPStat;
    my $popmsg=$pop->Message;
    $pop->Close;
    if ($alive eq "-1") {
        register_error("Authentication Failed<P>POP replied:<br>$popmsg");
    } else {
        return 1;  # We succeeded in authenticating
    }
}

#----------------------------------------------------------------------------\\
# auth_ftp: Authenticate against a FTP server.
#
# Parameters: username, password, servername.
#
# Returns: -1 if no connection can be made to the server
#          privilege level if successful
#          Does not return if username/password invalid.
#----------------------------------------------------------------------------//
sub auth_ftp {
    my ($user, $pass, $auth_svr) = @_;
    my $ftp = Net::FTP->new($auth_svr);
    if (!defined $ftp || $ftp eq '') {
        return -1;  # No handle; try next authentication server/method pair
    }
    $ftp->login($user, $pass);
    if (!$ftp->pwd()) {
        register_error("Authentication Failed<P>Bad Username or Password.");
    } else {
        return 1;  # We succeeded in authenticating
    }
}

#----------------------------------------------------------------------------\\
# auth_imap: Authenticate against an IMAP server.
#
# Parameters: username, password, servername.
#
# Returns: -1 if no connection can be made to the server
#          privilege level if successful
#          Does not return if username/password invalid.
#----------------------------------------------------------------------------//
sub auth_imap {
    my ($user, $pass, $auth_svr) = @_;
    my $imap = new Net::IMAP::Simple( $auth_svr );
    if (!defined $imap || $imap eq '') {
        return -1;  # No handle; try next authentication server/method pair
    }
    my $alive = $imap->login( $user, $pass );
    $imap->quit();
    if  (!$alive) {
        register_error("Authentication Failed");
    } else {
        return 1;  # We succeeded in authenticating
    }
}

#----------------------------------------------------------------------------\\
# auth_ldap: Authenticate against an LDAP server.
#
# Parameters: username, password, servername.
#
# Returns: -1 if no connection can be made to the server
#          privilege level if successful
#          Does not return if username/password invalid.
#----------------------------------------------------------------------------//
sub auth_ldap {
    my ($user, $pass, $auth_svr) = @_;
    my $dn = 'cn='.$user.",ou=people,$LDAP_BASE";
    my $ldap = Net::LDAPS->new($auth_svr);
    if (!defined $ldap || $ldap eq '') {
        return -1;  # Try next authentication server/method pair
    }
    # Attempt to bind as the given user.
    my $error = $ldap->bind (dn => $dn, password => $pass);

    if ($error->is_error) {
        my $ldapmsg = ldap_error_name($error->code);
        register_error('Registration Failed<P> Incorrect username/password.<br/>');
    }
    # Are you a valid normal user?
    if (defined $LDAP_USERS && $LDAP_USERS ne '') {
        my $result = $ldap->compare($LDAP_USERS,attr=>"uniqueMember",value=>$dn);
        if ($result->code != LDAP_COMPARE_TRUE ) {
            $ldap->unbind;
            register_error("Registration Failed<P>You are not in the list of users allowed to ".
                           "register machines on the network.<br>");
        } else {
             $ldap->unbind;
            return 1;  # We succeeded in authentication, and are in the users group
        }
    } else {
        $ldap->unbind;
        # Nothing to check against :-(
        register_error("LDAP user group not defined. Please check your Netreg configuration.<br />");
    }
}
#----------------------------------------------------------------------------/
1;
