#----------------------------------------------------------------------------\
# NetReg 2.0 (pre-release devel version)
#       DHCPD Registration Frontend
#
# Version 1.3: 
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
# Version 2.0:
# Portions Copyright (C) 1999-2002 Peter Valian, Southwestern University.
# Portions Copyright (C) 2002 David Bussenschutt, Griffith University, Australia.
# Portions Copyright (C) 2002 Robert Lowe,  Lawrence University, Wisconsin.
# Portions Copyright (C) 2002-2003 Michael McMaster, Griffith University, Australia.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Version 1.3: http://www.netreg.org/
# Version 2.0: http://www.sourceforge.net/projects/netreg/
#
# Version 1.3 code contributions by:
#   Peter Valian <valianp@southwestern.edu>,  Southwestern University.
#     Author of Version 1.3rc2 
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#   Michael Grinnell <grinnell@american.edu>, American University
#     LDAP Authentication
#
# Version 2.0 code contributions by:
#   Robert Lowe <robert.h.lowe@lawrence.edu>, Lawrence University, Wisconsin.
#     CIDR (Subnet Masks etc) Handling/Notation
#     Alpha Testing, and bug chasing in David's Code.
#   David Bussenschutt <D.Bussenschutt@mailbox.gu.edu.au>, Griffith University, Queensland, Australia.
#     Much re-arrangement, and fixes, including but not limited to:
#     Perl 5 Syntax, and modules
#     File Locking
#     OMAPI Support
#     Remote dhcpdmanager daemon, and client support (functionally similar to omapi, but more fully implemented)
#     Improved Error Handling
#     Use of -w for improved code
#     LDAP group handling, for Permissions.
#     install_script
#   Michael McMaster <m4k3r@users.sourceforge.net>, Griffith University,  Queensland, Australia.
#     DBI/Oracle Support
#     HTML extraction/templating
#     Permissions handling (access controls)
#     Cookie based authentication
#     Interface for editing dhcp options/groups and hosts.
#     Synchronisation between hosts
#     Scalability improvements
#     Various code clean-ups
#     Consistency checker for host information 
#----------------------------------------------------------------------------/
package Files_Remote;
use strict;
use warnings;
#----------------------------------------------------------------------------\\
#
# Exports
#
#----------------------------------------------------------------------------//
use Exporter ();
our @ISA = qw (Exporter);
our @EXPORT = ( );
our @EXPORT_OK = qw (  read_conf_new_remote append_conf_remote
                       get_host_info_remote db_init
                       delete_conf_new_remote
                       read_leases_remote
                       status_dhcpd_remote get_host_info_remote_all 
                       read_ip_leases_remote get_all_servers);

#----------------------------------------------------------------------------\\
#
# Imported Modules
#
#----------------------------------------------------------------------------//
# If using mod_perl, the $FindBin::RealBin directory will be set ONCE only
# see startup.pl
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.

# Netreg modules
use Variables qw ( $KeyName $Key $DHCPDMANAGER $DEBUG $PING );
use Html qw( admin_error );
use Subnets qw( what_is_my_registered_subnet );
use NetregMisc    qw( append_error_log );



use lib $DHCPDMANAGER;
use DhcpdManager; 

#----------------------------------------------------------------------------\\
#
# Constants
#
#----------------------------------------------------------------------------//
# Optional mod_perl support.
# Sets a constant, and ensures the correct exit() function is called.
use constant IS_MODPERL => $ENV{MOD_PERL};
use subs qw(exit);
# Select the correct exit function
*exit = IS_MODPERL ? \&Apache::exit : sub { CORE::exit };

#----------------------------------------------------------------------------\\
#
# Global Variables
#
# These are shared within this perl module only.
# These variables are specifically shared between executions if using mod_perl
#----------------------------------------------------------------------------//
# A hash of dhcpdmanager server IP's to connection handles.
# This enables persistent connections to remote servers
my %connections = ();

#----------------------------------------------------------------------------\\
#
# SQL init
#
#----------------------------------------------------------------------------//
# Prepare database queries. This function is only called by db_connect in
# Database.pm

my ($sql_all_servers,$sql_servers1,$sql_servers2);
sub db_init {
    my $dbh = shift;

    append_error_log("db_init: start") if $DEBUG >=1;

    $sql_all_servers = $dbh->prepare("select distinct server_ip from campuses");

    # Cannot use table joins. This seems to defeat the purpose of having a 
    # relational database, but the CSV driver doesn't currently support it.
    # Neither do subqueries.
     $sql_servers1 = $dbh->prepare("select campus from subnets where ".
                                   "cidr_subnet = ? and cidr_mask = ?");
     $sql_servers2 = $dbh->prepare("select distinct server_ip from campuses ".
                                   "where campus = ?");

    append_error_log("db_init: end") if $DEBUG >=1;

}

#----------------------------------------------------------------------------\\
#
# Exported Methods
#
#----------------------------------------------------------------------------//

#----------------------------------------------------------------------------\\
# status_dhcpd_remote: 
# call the dhcpdmanager server, to check it's working, and also have it check
# the status of the remote dhcpd server (eg return it's current PID).
#
# Parameters: 
#             (optional) an IP. If provided only the subnets relevant to the
#                 IP will be contacted.
#
# Returns: an array reference to (server_ip,status,pid,misc), or undef on error. 
#----------------------------------------------------------------------------//
sub status_dhcpd_remote {
    my $tmpIP = shift || '';

    my @statii = ();

    append_error_log("status_dhcpd_remote: start") if $DEBUG >=1;

    # figure out dhcpd servers that apply... 

    append_error_log("status_dhcpd_remote: about to lookup servers") if $DEBUG >=1;
    my $dhcpdservers = get_all_servers($tmpIP);
    return undef unless $dhcpdservers;
    my @dhcpdservers = @$dhcpdservers;

    append_error_log("status_dhcpd_remote: about to query servers") if $DEBUG >=2;
    foreach my $ServerIP ( @dhcpdservers) {
        append_error_log("status_dhcpd_remote: querying server: $ServerIP") if $DEBUG >=2;

        $ServerIP =~ s/\/.*$//; # remove the auth method, if defined.

        # open up the TCPIP socket etc... 
        # '1' here means don't display connection errors immediately to the user, 0 means do
        my $conn = connect_dhcpdmanager($ServerIP, "1");
	push @statii, "$ServerIP,DOWN,unknown,Connect Error" unless defined $conn;
        next unless defined $conn;

        # request status of dhcpd server
        $conn->send("return dhcpd server status");
        my $err =  $conn->recv();
        $err=undef unless $err =~ /^OK/;
        
	push @statii, "$ServerIP,DOWN,unknown,Unrecognised response" unless defined $err;
        next unless defined $err;
        
        $err =~ s/^OK[\s-]+//;  # remote the OK from the start of the response
        push @statii, "$ServerIP,$err";
    }

    append_error_log("status_dhcpd_remote: end") if $DEBUG >=1;

    return \@statii;
}

#----------------------------------------------------------------------------\\
# delete_conf_new_remote:
# Deletes the given mac from the configs of remote dhcpd servers.
#
# Parameters: 
#             the MAC to delete
#
# Returns: undef on error, 0 on remote error or 1 on success.
#----------------------------------------------------------------------------//
sub delete_conf_new_remote {
    my $mac = shift;

    append_error_log("delete_conf_new_remote: start") if $DEBUG >=1;

    # figure out dhcpd servers that apply...
    my $dhcpdservers = get_all_servers();
    return undef unless $dhcpdservers;
    my @dhcpdservers = @$dhcpdservers; 

    my $ret;
    foreach my $ServerIP ( @dhcpdservers) {
        $ServerIP =~ s/\/.*$//; # remove the auth method, if defined.

        # open up the TCPIP socket etc...
        my $conn = connect_dhcpdmanager($ServerIP);
        admin_error("failed to connect to dhcpdmanager server (at: '$ServerIP'): $! \n",
                    "Network5")  unless $conn;
        next unless defined $conn;

        # request a delete of this host record.
        $conn->send("delete host with mac of $mac");
        my $err =  $conn->recv();
 
        # getting an OK response is fine.
        # failure to delete a MAC that doesn't exist is not an error (as far as I'm concerned)
        if ( $err =~ /^OK/ || $err =~ /already\s+does\s+not\s+exist/o ) { 
            $err=0;
        } else { 
            # other things are!
            admin_error("failed to delete from $ServerIP: $err \n","Network"); 
            $err=1; 
        }

        $ret++ if $err;        
      }

    append_error_log("delete_conf_new_remote: end") if $DEBUG >=1;


      return !$ret;
}

#----------------------------------------------------------------------------\\
# read_ip_leases_remote:
# Retuns a list of all unique IP's currently leased.
#
# Parameters: None.
#
# Returns: an array reference of all leased IP's, or undef on error
#----------------------------------------------------------------------------//
sub read_ip_leases_remote {
    my @leases=();      # An array of IP's

    append_error_log("read_ip_leases_remote: start") if $DEBUG >=1;

    # figure out dhcpd servers that apply...
    my $dhcpdservers = get_all_servers();
    return undef unless $dhcpdservers;
    my @dhcpdservers = @$dhcpdservers;

    # connect to all the servers and send data.
    # DO NOT wait for the server to reply before moving to the
    # next server.
    my @connections = ();
    foreach my $ServerIP ( @dhcpdservers ) {
        $ServerIP =~ s/\/.*$//; # remove the auth method, if defined.
        # open up the TCPIP socket etc...
        my $conn = connect_dhcpdmanager($ServerIP);
        admin_error("failed to connect to dhcpdmanager server (at: '$ServerIP'): $! \n","Network1")  unless $conn;
        next unless defined $conn;

       # get a list of all the 'host' entries in the remote server
       $conn->send("list ips");
       push @connections,$conn;
    }

    # Read the results from each server.
    foreach my $conn (@connections) {
        my @tmp =  $conn->read_data();
        push @leases, @tmp;
    }

    # remove duplicates from the list.
    # this strange code is straight from the Perl Cookbook (Orielly).
    my %seen = ();
    my @unique_ips = grep { ! $seen{$_} ++ } @leases;

    append_error_log("read_ip_leases_remote: end") if $DEBUG >=1;

    return \@unique_ips;
}

#----------------------------------------------------------------------------\\
# read_leases_remote:
# Retuns a list of all current leases.
#
# Parameters: 
#             an (optional) IP. If provided, only servers relevant to the
#                 given IP will be contacted.
#             an (optional) subnet. If provided, only leases in the given
#                 subnet will be returned.
#             an (optional) query. If provided, only leases matching the
#                 given query will be returned.
#
#             A subnet AND query may NOT be provided simultaneously.
#
# Returns: an array reference of all leases, or undef on error
#----------------------------------------------------------------------------//
sub read_leases_remote {
    my $tmpIP = shift;
    my $subnet = shift;
    my $query = shift;

    append_error_log("read_leases_remote: start") if $DEBUG >=1;

    my @leases=();      # an array of leases.

    # figure out dhcpd servers that apply...
    my $dhcpdservers = get_all_servers($tmpIP);
    return undef unless $dhcpdservers;
    my @dhcpdservers = @$dhcpdservers;    

    # Create a connection to each server, but DO NOT wait to read data.
    my @connections = ();
    foreach my $ServerIP ( @dhcpdservers ) {
        $ServerIP =~ s/\/.*$//o; # remove the auth method, if defined.

        # open up the TCPIP socket etc...
        my $conn = connect_dhcpdmanager($ServerIP);
        admin_error("failed to connect to dhcpdmanager server (at: '$ServerIP'): $! \n","Network2")  unless $conn;
        next unless defined $conn;

        # get a list of all the 'host' entries in the remote server
        if ($subnet) {
            $conn->send("list leases in $subnet");
        } elsif ($query) {
            $conn->send("list leases match $query");
        } else {
            $conn->send("list leases");
        }
        push @connections, $conn;
    }
    # Read the data back from all servers.
    foreach my $conn (@connections) {
        my @tmp =  $conn->read_data();
        push @leases, @tmp;
    }
    # remove duplicates from the list.
    # this strange code is straight from the Perl Cookbook (Orielly).
    my %seen = ();
    my @unique_leases = grep { ! $seen{$_} ++ } @leases;

    append_error_log("read_leases_remote: end") if $DEBUG >=1;

    return \@unique_leases;
}

#----------------------------------------------------------------------------\\
# read_conf_new_remote:
# Retuns all entries in the dhcpd.conf.new files on all dhcpd servers. 
#
# Parameters: 
#             an (optional) IP. If provided, only servers relevant to the
#                 given IP will be contacted.
#
# Returns: an array reference of all conf entries, or undef on error
#----------------------------------------------------------------------------//
sub read_conf_new_remote {
    my $tmpIP = shift || '';

    append_error_log("read_conf_new_remote: start") if $DEBUG >=1;

    my %hoststuff;

    # figure out dhcpd servers that apply...
    my $dhcpdservers = get_all_servers($tmpIP);
    return undef unless $dhcpdservers;
    my @dhcpdservers = @$dhcpdservers;

    foreach my $ServerIP ( @dhcpdservers ) {
        my $hosts = list_hosts_remote($ServerIP);
        foreach my $hostrecord ( @$hosts ) {
	    $hoststuff{$hostrecord}++;
        }
    }

    append_error_log("read_conf_new_remote: end") if $DEBUG >=1;

    my @ret =  sort keys %hoststuff;
    return \@ret;
}

#----------------------------------------------------------------------------\\
# append_conf_remote:
# Append the given information to the dhcpd.conf.new file on a remote server.
#
# Parameters: A server running the dhcpdmanager daemon
#             An array of lines to append.
#
# Returns: undef on error, 0 on remote error, 1 on success
#----------------------------------------------------------------------------//
sub append_conf_remote {
    my $ServerIP = shift;
    my @appendlines = @_;

    append_error_log("append_conf_remote: start") if $DEBUG >=1;

    $ServerIP =~ s/\/.*$//; # remove the auth method, if defined.

    # open up the TCPIP socket etc...
    my $conn = connect_dhcpdmanager($ServerIP);
    admin_error("failed to connect to dhcpdmanager server (at: '$ServerIP'): $! \n","Network3")  unless $conn;
    return undef unless defined $conn;

    my $e = 0;
    foreach (@appendlines) {
        # add the 'host' record to the dhcpd server...
        $conn->send("add $_");  # send the add request to the server...
        $e++ unless $conn->recv() =~ /^OK/;
    }

    append_error_log("append_conf_remote: end") if $DEBUG >=1;

    return undef if $e>0;
}

#----------------------------------------------------------------------------\\
# get_host_info_remote:
# Attempt to find the MAC address for a given IP on a given Server.
#
# Parameters: An IP to lookup 
#             A server running the dhcpdmanager daemon
#
# Returns: undef on error, 0 on remote error, MAC address on success
#----------------------------------------------------------------------------//
sub get_host_info_remote {
    my ($IP,$ServerIP) = @_;
 
    append_error_log("get_host_info_remote: start") if $DEBUG >=1;

    $ServerIP =~ s/\/.*$//; # remove the auth method, if defined.

    # open up the TCPIP socket etc...
    my $conn = connect_dhcpdmanager($ServerIP);
    admin_error("failed to connect to server '$ServerIP' (via dhcpdm): $! \n","Network")  unless $conn;
    return undef unless defined $conn;

    # get a list of all the 'host' entries in the remote server
    $conn->send("return mac address for $IP");
    my $MAC =  $conn->recv();

    append_error_log("get_host_info_remote: end") if $DEBUG >=1;

    return '0' if $MAC =~ /^ERR[\s-]+mac\s+address\s+not\s+found/i;
    return uc($MAC);
}

#----------------------------------------------------------------------------\\
# get_host_info_remote_all:
# Attempt to find the MAC address for a given IP on a set of servers.
#
# Parameters: 
#             An IP to lookup
#
# Returns: undef on error, 0 on remote error, MAC address on success
#----------------------------------------------------------------------------//
sub get_host_info_remote_all {
    my $tmpIP = shift; 
    my %host_info;

    append_error_log("get_host_info_remote_all: start") if $DEBUG >=1;

    # figure out dhcpd servers that apply...
    my $dhcpdservers = get_all_servers($tmpIP);
    return undef unless $dhcpdservers;
    my @dhcpdservers = @$dhcpdservers;

    foreach my $ServerIP ( @dhcpdservers ) {
        my $mac = get_host_info_remote($tmpIP,$ServerIP);

        # no error's here, assume a null array returned means no hosts stored in remote server.
        admin_error ("Unable to contact $ServerIP for host query","Network") unless defined $mac;

        $host_info{$mac}++ if $mac;
    }

    my @macs = keys %host_info;
    my $message = join (", ",@macs);
    admin_error ("Multiple MAC's for one IP: $message","Consistency") unless $#macs <= 0;

    append_error_log("get_host_info_remote_all: end") if $DEBUG >=1;

    return $macs[0] if defined $macs[0];
    return '0'; 
}

#----------------------------------------------------------------------------\\
#
# Non-Exported Methods
#
#----------------------------------------------------------------------------//
sub pingcheck {
my $ip = shift;
return undef unless $ip;
  append_error_log("pingcheck: start") if $DEBUG >=1;
  my $pingcmd;
  # determine which type of ping we are using?
  my $pingtype = `$PING -c 1 -w 1 -n -q localhost 2>&1 `;
  if ( $pingtype =~ /usage/i ) {
    $pingcmd = "$PING $ip 1 > /dev/null";
  } else {
    $pingcmd = "$PING -c 1 -w 1 -n -q $ip > /dev/null"; 
  }
  my $status = system ($pingcmd);
  my $ret = ($status == 0?"1":"");
  return $ret;
}

#----------------------------------------------------------------------------\\
# connect_dhcpdmanager:
# Attempts to make a connection to the given dhcpdmanager IP, unless a connection
# already exists.
#
# Parameters: Server IP
#             No_errors - true if this function shouldn't throw errors
#
# Returns: A connection handle or undef.
#----------------------------------------------------------------------------//
sub connect_dhcpdmanager {
    my $server = shift;
    my $no_errors = shift;

    append_error_log("connect_dhcpdmanager: start") if $DEBUG >=1;

    #if ($connections{$server} && $connections{$server}->ping()) {
    # TODO: the above line is "safer" and more correct, but the ping is also 
    #        really slow, and makes the web pages take forever when there are connection probs.
    # so we currently have this line instead.....
    if ($connections{$server}) {

        return $connections{$server};
    }

    my $ping = pingcheck($server);
    if (!$ping) {  
        admin_error("Could not PING server $server", "Network") unless $no_errors;
	return undef;
    }
    my $conn = DhcpdManager->new(
                            server => $server,
                            omapi_key => $KeyName,
                            omapi_secret => $Key,
                            ); # open TCPIP
    append_error_log("connect_dhcpdmanager: after ->new call") if $DEBUG >=2;
    if (!$conn) {
        admin_error("Could not open connection to dhcpdmanager at $server", "Network")
            unless $no_errors;
	# TODO: do we need to exit here really? exiting seems to speed up the webpage/s
        # when there are remote dhcpdmanager problems (eg not running at all), but it also 
        # prevents us from getting ANYTHING useful from the webpage, so it might be better
        # if the calling subs handled the 'undef' return better.
	#exit ;
        return undef;
    }

    append_error_log("connect_dhcpdmanager: before CONNECT") if $DEBUG >=2;
    # send initial CONNECT, to initiate server response
    $conn->send("CONNECT");

    append_error_log("connect_dhcpdmanager: before ->auth()") if $DEBUG >=2;
    # handshake with the server
    my $ret = $conn->auth();
    if (!$ret) {
        admin_error("Could not authenticate to dhcpdmanager at $server", "Authentication")
            unless $no_errors;
        return undef;
    }
    $connections{$server} = $conn;
    append_error_log("connect_dhcpdmanager: end") if $DEBUG >=1;
    return $conn;
}

#----------------------------------------------------------------------------\\
# list_hosts_remote:
# Returns a list of all host entries on a remote DHCPD server.
#
# Parameters: Server IP
#
# Returns: An array reference to the host list, or undef on error. 
#----------------------------------------------------------------------------//
sub list_hosts_remote {
    my $ServerIP = shift;
    my @hosts=();
    append_error_log("list_hosts_remote: start") if $DEBUG >=1;

    $ServerIP =~ s/\/.*$//;

    # open up the TCPIP socket etc...
    my $conn = connect_dhcpdmanager($ServerIP);
    admin_error("failed to connect to dhcpdmanager server (at: '$ServerIP'): $!","Network4")  unless $conn;
    return undef unless defined $conn;

    # get a list of all the 'host' entries in the remote server
    $conn->send("list hosts");
    push @hosts, $conn->read_data();

    append_error_log("list_hosts_remote: end") if $DEBUG >=1;

    return \@hosts;
}

#----------------------------------------------------------------------------\\
# get_all_servers:
# Attempt to find the MAC address for a given IP on a set of servers.
#
# Parameters: 
#             An IP to lookup
#
# Returns: undef on error, 0 on remote error, MAC address on success
#----------------------------------------------------------------------------//
sub get_all_servers {
    my $tmpIP = shift;
    # figure out dhcpd servers that apply...
    my $sql_executed;
    if ($tmpIP) {
        my ($subnet, $mask)  = what_is_my_registered_subnet($tmpIP);
        return undef unless $subnet;
        $sql_servers1->execute($subnet, $mask);
        my @subnet_row = $sql_servers1->fetchrow_array();
        $sql_servers1->finish();
        $sql_servers2->execute($subnet_row[0]);
        $sql_executed = $sql_servers2;
    } else {
        # findout a list of EVERY single dhcpd server available
        $sql_all_servers->execute();
        $sql_executed = $sql_all_servers;
    }
    my @dhcpdservers;
    while (my @row = $sql_executed->fetchrow_array) {
        push @dhcpdservers, $row[0];
    }
    $sql_executed->finish();
    return \@dhcpdservers;
}
#----------------------------------------------------------------------------/
1;
