#----------------------------------------------------------------------------\
# NetReg 2.0 (pre-release devel version)
#       DHCPD Registration Frontend
#
# Version 1.3: 
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
# Version 2.0:
# Portions Copyright (C) 1999-2002 Peter Valian, Southwestern University.
# Portions Copyright (C) 2002 David Bussenschutt, Griffith University, Australia.
# Portions Copyright (C) 2002 Robert Lowe,  Lawrence University, Wisconsin.
# Portions Copyright (C) 2002-2003 Michael McMaster, Griffith University, Australia.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Version 1.3: http://www.netreg.org/
# Version 2.0: http://www.sourceforge.net/projects/netreg/
#
# Version 1.3 code contributions by:
#   Peter Valian <valianp@southwestern.edu>,  Southwestern University.
#     Author of Version 1.3rc2 
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#   Michael Grinnell <grinnell@american.edu>, American University
#     LDAP Authentication
#
# Version 2.0 code contributions by:
#   Robert Lowe <robert.h.lowe@lawrence.edu>, Lawrence University, Wisconsin.
#     CIDR (Subnet Masks etc) Handling/Notation
#     Alpha Testing, and bug chasing in David's Code.
#   David Bussenschutt <D.Bussenschutt@mailbox.gu.edu.au>, Griffith University, Queensland, Australia.
#     Much re-arrangement, and fixes, including but not limited to:
#     Perl 5 Syntax, and modules
#     File Locking
#     OMAPI Support
#     Remote dhcpdmanager daemon, and client support (functionally similar to omapi, but more fully implemented)
#     Improved Error Handling
#     Use of -w for improved code
#     LDAP group handling, for Permissions.
#     install_script
#   Michael McMaster <m4k3r@users.sourceforge.net>, Griffith University,  Queensland, Australia.
#     DBI/Oracle Support
#     HTML extraction/templating
#     Permissions handling (access controls)
#     Cookie based authentication
#     Interface for editing dhcp options/groups and hosts.
#     Synchronisation between hosts
#     Scalability improvements
#     Various code clean-ups
#     Consistency checker for host information 
#----------------------------------------------------------------------------/
package Html;
use strict;
use warnings;
#----------------------------------------------------------------------------\\
#
# Exports
#
#----------------------------------------------------------------------------/
use Exporter;
our @ISA = qw (Exporter);

our @EXPORT = qw ( );
our @EXPORT_OK = qw( admin_error register_error print_success print_register_page
		     set_cookie get_cookies print_html_file html_init get_input );
#----------------------------------------------------------------------------\\
#
# Imported Modules
#
#----------------------------------------------------------------------------//
# If using mod_perl, the $FindBin::RealBin directory will be set ONCE only
# see startup.pl
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.

use IO::File;   # Used to enable the recursive print_html_file function.

# Netreg modules
use NetregMisc qw (append_error_log);
use Variables qw( $IP $GFX $SCRIPTS $HTML $ADMINPATH $HELP_EXTN );

#----------------------------------------------------------------------------\\
#
# Constants
#
#----------------------------------------------------------------------------//
# Optional mod_perl support.
# Sets a constant, and ensures the correct exit() function is called.
use constant IS_MODPERL => $ENV{MOD_PERL};
use subs qw(exit);
# Select the correct exit function
*exit = IS_MODPERL ? \&Apache::exit : sub { CORE::exit };

#----------------------------------------------------------------------------\\
#
# Global Variables
#
# These are shared within this perl module only.
# These variables are specifically shared between executions if using mod_perl
#----------------------------------------------------------------------------//
# The following variables are undefined unless html_init is called at the start
# of any executable script!
my $is_header_printed = "0";   # set to true when a valid HTML header is printed.
my $is_content_type_printed = "0";
my %FORM = ();                 # set to the contents of a CGI form.

#----------------------------------------------------------------------------\\
#
# Exported Methods
#
#----------------------------------------------------------------------------//

#----------------------------------------------------------------------------\\
# html_init: Sets up this module for use.
#
# Parameters: None.
#
# Returns: nothing useful
#----------------------------------------------------------------------------//
sub html_init {
    $is_header_printed = $is_content_type_printed = 0;
    %FORM = ();
}

#----------------------------------------------------------------------------\\
# get_input: Get the input from a CGI form using both post and get methods.
# Also saves the URI of the current page.
#
# Parameters: None.
#
# Returns: A hash of form elements.
#----------------------------------------------------------------------------//
sub get_input {
    return %FORM if %FORM;

    # Remember the filename of the URL URL that we visited from (if any):
    my $t = $ENV{'REQUEST_URI'} || $ENV{'SCRIPT_FILENAME'} || '';
    # strip off any leading path..
    $t  =~ s/^[\/\w\-0-9\.]*\///;
    $FORM{'uri'} = $t || '/';

    my @pairs = ();
    if ($ENV{'QUERY_STRING'}) {
        @pairs = split(/&/, $ENV{'QUERY_STRING'});
    }
    if ($ENV{'CONTENT_LENGTH'}) {
        my $buffer;
        read(STDIN,$buffer, $ENV{'CONTENT_LENGTH'});
        push @pairs,split(/&/o,$buffer);   
    }
    foreach my $pair (@pairs) {
        my ($name, $value) = split(/=/, $pair);
        $value =~ tr/+/ /;
        $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/ego;
        $value =~ s/[\>\<]//g; # Remove any nasty characters
        $FORM{$name} = $value;
    }
    return %FORM;
}

#----------------------------------------------------------------------------\\
# print_html_file: Reads the given file and prints it to stdout.
# Replaces global HTML variables with their content.
# May be called with values for substition (key/value pairs)
#
# Parameters: Filename to print
#             key/value substitution pairs
#
# Returns: nothing useful.
#----------------------------------------------------------------------------//
sub print_html_file {
    my $file = shift;
    my @subs = @_;

    my %form = get_input();
    my $uri = $form{'uri'};
    my $base = $uri;
    $base =~ s/\?.*//g; # get rid of anything after ?
 
    return 1 if ($is_header_printed && $file =~ /header/);
    $is_header_printed = "1";
    content_type();

    my $filehandle;
    if (!(open $filehandle, "<$HTML/$file")) {
        admin_error ("Unable to open $file: $!");
        exit;
    }
  
    while (my $line = <$filehandle>) {
        if ($line =~ /#include\s+(.+)/) {
            print_html_file($1);
        } else {
            $line =~ s/\$GFX/$GFX/g; 
            $line =~ s/\$ADMINPATH/$ADMINPATH/g; 
            $line =~ s/\$HELP_EXTN/$HELP_EXTN/g; 
            $line =~ s/\$SCRIPTS/$SCRIPTS/g; 
            $line =~ s/\^URI/$uri/g; 
            $line =~ s/\^BASEURI/$base/g;
            for (my $count = 0; $count < $#subs; $count += 2) {
               my $key = $subs[$count];
               my $value = $subs[$count+1] || '';
               $line =~ s/$key/$value/g;
            } 
            print $line;
        }
    }
    close $filehandle;
    exit if ($file =~ /footer/);
}

#----------------------------------------------------------------------------\\
# set_cookie: Sends a cookie to the web browser. Requires that this is used before 
# sending any other data to the browser. 
# Set a undef $expiration to create a "session" cookie (not stored on disk).
#
# Parameters: Name of the cookie
#             Cookie value
#             Cookie expiration (or undef for a session cookie)
#
# Returns: nothing useful
#----------------------------------------------------------------------------//
sub set_cookie {
    my ($name, $value, $expiration) = @_;
   
    if ($is_content_type_printed) {
        print STDERR "ERROR: Cookie printed AFTER other HTTP headers\n";
        print STDERR "Your code is incorrect.\n";
        exit;
    }
    chomp $name; 
    chomp $value; 
    chomp $expiration if $expiration;
  
    my $cookieString = "$name=$value; path=/; ";
    $cookieString = $cookieString."expires=$expiration;" if $expiration;

    if (!IS_MODPERL) {
        print "Content-type: text/html\n";
        print "Set-Cookie: ".$cookieString."\r\n\r\n";
    } else {
        my $r = Apache->request();
        $r->content_type("text/html");
        $r->header_out('Set-Cookie',$cookieString);
        $r->send_http_header;
    }
    $is_content_type_printed = "1";
}

#----------------------------------------------------------------------------\\
# get_cookies: Returns a hash of key/value pairs from the browser.
#
# Parameters: None
#
# Returns: A hash of cookie values
#----------------------------------------------------------------------------//
sub get_cookies {
    return ('key', 'value') unless $ENV{'HTTP_COOKIE'};

    my @rawCookies = split (/; /,$ENV{'HTTP_COOKIE'});
    my %cookies;
   
    #TODO: it is quite legal for there to exist MULTIPLE
    # cookies with the same 'name' (or 'key').
    # when we store these cookies into a hash, we are
    # effectively PREVENTING this from occuring, and 
    # we only get ONE cookie into the hash...it might be the wrong one!

    foreach (@rawCookies) {
        my ($key,$val) = split (/=/,$_);
    	$cookies{$key} = $val;
    }

    return %cookies;
}

#----------------------------------------------------------------------------\\
# register_error: Displays an error message to the user, and logs and error.
#
# Parameters: An error message.
#
# Returns: Does NOT return.
#----------------------------------------------------------------------------//
sub register_error {
    my $error_msg = $_[0] || ('unknown error');

    content_type();

    my @error = ( "$error_msg\n");
    append_error_log(@error);

    if (!$is_header_printed) {
      if (defined $FORM{'uri'} && $FORM{'uri'} =~ /admin/) {
          print_html_file("admin_header.html");
      } else {
          print_html_file("register_header.html");
      }
    }
    
    print_html_file("register_error.html", "\\^ERROR_MSG", $error_msg);

    if (defined $FORM{'uri'} && $FORM{'uri'} =~ /admin/) {
        print_html_file("admin_footer.html");
    } else {
        print_html_file("register_footer.html");
    }
    exit;
}

#----------------------------------------------------------------------------\\
# admin_error: Displays an error message to the user, and logs and error.
#
# Parameters: An error message.
#             The type of error
#
# Returns: Nothing Useful.
#----------------------------------------------------------------------------//
sub admin_error {
    my $error_msg = shift;
    my $error_type = shift || 'Script';

    my %FORM = get_input();
    content_type();
    if (!$is_header_printed) {
        if ( defined $FORM{'uri'} && $FORM{'uri'} =~ /admin/ ) {
            print_html_file("admin_header.html");
        } else {
            print_html_file("register_header.html");
        }
    }

    my @error = ( "$error_msg\n","$error_type\n\n");
    append_error_log(@error);
    
    print_html_file("admin_error.html", "\\^ERROR_MSG", $error_msg,
                    "\\^ERROR_TYP", $error_type);
}

#----------------------------------------------------------------------------\\
# print_success: Displays an message to the user that a registration succeeded.
#
# Parameters: None
#
# Returns: Nothing Useful.
#----------------------------------------------------------------------------//
sub print_success {
  
    #dhcpd servers refresh every minute... 
    my ($sec) = localtime;
    my $refresh = (abs($sec-=60))+2;
    $sec = (abs($sec));

    print "<h2>Registration Complete!</h2><br>Please wait at least $sec seconds ";
    print "(for the server to handle you request) and then<br>reboot ";
    print "your computer</b>.<P>";
    print_html_file("register_footer.html");
}


#----------------------------------------------------------------------------\\
# print_register_page: Displays the main part of the registration front page.
#
# Parameters: 1) A Privilege level.
#             2) A list of valid domains that the user may register for. 
#
# Returns: Nothing Useful.
#----------------------------------------------------------------------------//
sub print_register_page {
    my $priv = shift;
    my @domains = sort @_;

    content_type();

    my $host_entry = '';
    if ($priv) {
        $host_entry = "<tr><td align=right>Optional host name:<br />Leave blank for default</td>".
                      "<td><input type=text name=hostname size=30></td></tr>";
    }
  
    my $domain_options = '';
    foreach (@domains) {
    	$domain_options .= "<option value=\"$_\">$_ </option>\n";
    }

    print_html_file("register_page.html", "\\^HOSTNAME", $host_entry,
                                          "\\^OPTIONS", $domain_options);
    print_html_file("register_footer.html");
}

#----------------------------------------------------------------------------\\
#
# Non-Exported Methods
#
#----------------------------------------------------------------------------//

#----------------------------------------------------------------------------\\
# content_type: Prints a html content type to the page in the case of a
# wierd error.
#
# Parameters: nothing.
#
# Returns: Nothing useful.
#----------------------------------------------------------------------------//
sub content_type {
    print "Content-type: text/html\r\n\r\n" unless $is_content_type_printed;
    $is_content_type_printed = "1";
}

#----------------------------------------------------------------------------/
1;
