#----------------------------------------------------------------------------\
# NetReg 2.0 (pre-release devel version)
#       DHCPD Registration Frontend
#
# Version 1.3: 
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
# Version 2.0:
# Portions Copyright (C) 1999-2002 Peter Valian, Southwestern University.
# Portions Copyright (C) 2002 David Bussenschutt, Griffith University, Australia.
# Portions Copyright (C) 2002 Robert Lowe,  Lawrence University, Wisconsin.
# Portions Copyright (C) 2002-2003 Michael McMaster, Griffith University, Australia.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Version 1.3: http://www.netreg.org/
# Version 2.0: http://www.sourceforge.net/projects/netreg/
#
# Version 1.3 code contributions by:
#   Peter Valian <valianp@southwestern.edu>,  Southwestern University.
#     Author of Version 1.3rc2 
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#   Michael Grinnell <grinnell@american.edu>, American University
#     LDAP Authentication
#
# Version 2.0 code contributions by:
#   Robert Lowe <robert.h.lowe@lawrence.edu>, Lawrence University, Wisconsin.
#     CIDR (Subnet Masks etc) Handling/Notation
#     Alpha Testing, and bug chasing in David's Code.
#   David Bussenschutt <D.Bussenschutt@mailbox.gu.edu.au>, Griffith University, Queensland, Australia.
#     Much re-arrangement, and fixes, including but not limited to:
#     Perl 5 Syntax, and modules
#     File Locking
#     OMAPI Support
#     Remote dhcpdmanager daemon, and client support (functionally similar to omapi, but more fully implemented)
#     Improved Error Handling
#     Use of -w for improved code
#     LDAP group handling, for Permissions.
#     install_script
#   Michael McMaster <m4k3r@users.sourceforge.net>, Griffith University,  Queensland, Australia.
#     DBI/Oracle Support
#     HTML extraction/templating
#     Permissions handling (access controls)
#     Cookie based authentication
#     Interface for editing dhcp options/groups and hosts.
#     Synchronisation between hosts
#     Scalability improvements
#     Various code clean-ups
#     Consistency checker for host information 
#----------------------------------------------------------------------------/
package Option;
use strict;
use warnings;

#----------------------------------------------------------------------------\\
#
# Imported Modules
#
#----------------------------------------------------------------------------//
use Html qw (print_html_file);

#----------------------------------------------------------------------------\\
#
# Object Methods
#
#----------------------------------------------------------------------------//

#----------------------------------------------------------------------------\\
# new: Creates a new "Option" object. (Stores info about dhcp options).
#
# Parameters: The name of this option.
#             The value of the option (including any "" if necessary)
#             True if the option isn't prefixed with the "option" string 
#
# Returns: An Option object (a blessed hash reference).
#----------------------------------------------------------------------------//
sub new {
    my ($class,$name,$value,$fakeOption) = @_;
    my $self  = {}; 

    $self->{'name'} = $name; 
    $self->{'value'} = $value;
    $self->{'fakeOption'} = $fakeOption;

    bless($self, $class);    
    return $self;
}   

#----------------------------------------------------------------------------\\
# display: Display the contents of this option as HTML.
#
# Parameters: The cumulatative group id string.
#             A saved md5 hash of the dhcp conf file
#
# Returns: Nothing useful.
#----------------------------------------------------------------------------//
sub display {
    my ($self,$url,$saved_md5) = @_;

    my $value = $self->{'value'};
    $value =~ s/\"/\&\#34\;/g;
    print_html_file("static_option_row.html", "\\^SAVED_MD5", $saved_md5,
                    "\\^URL", $url, "\\^OPTION", $self->{'name'}, "\\^OPTVAL", $value);

}

#----------------------------------------------------------------------------\\
# write: write the contents of this option to a given file handle in dhcpd.conf
# format.
#
# Parameters: A filehandle to write to.
#
# Returns: Nothing useful.
#----------------------------------------------------------------------------//
sub write {
    my ($self,$fh) = @_;

    print $fh "option " unless $self->{'fakeOption'};
    print $fh $self->{'name'}." ".$self->{'value'}.";\n";
}

#----------------------------------------------------------------------------\\
# name: returns the name of this option.
#
# Parameters: None.
#
# Returns: The name of this option.
#----------------------------------------------------------------------------//
sub name {
    my $self = shift;
    return $self->{'name'};
}

#----------------------------------------------------------------------------\\
# update: Assigns a new value to this option.
#
# Parameters: The new value
#
# Returns:  Nothing useful.
#----------------------------------------------------------------------------//
sub update {
    my ($self,$value) = @_;
    $self->{'value'} =  $value;
}

1;
