#----------------------------------------------------------------------------\
# NetReg 2.0 (pre-release devel version)
#       DHCPD Registration Frontend
#
# Version 1.3: 
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
# Version 2.0:
# Portions Copyright (C) 1999-2002 Peter Valian, Southwestern University.
# Portions Copyright (C) 2002 David Bussenschutt, Griffith University, Australia.
# Portions Copyright (C) 2002 Robert Lowe,  Lawrence University, Wisconsin.
# Portions Copyright (C) 2002-2003 Michael McMaster, Griffith University, Australia.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Version 1.3: http://www.netreg.org/
# Version 2.0: http://www.sourceforge.net/projects/netreg/
#
# Version 1.3 code contributions by:
#   Peter Valian <valianp@southwestern.edu>,  Southwestern University.
#     Author of Version 1.3rc2 
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#   Michael Grinnell <grinnell@american.edu>, American University
#     LDAP Authentication
#
# Version 2.0 code contributions by:
#   Robert Lowe <robert.h.lowe@lawrence.edu>, Lawrence University, Wisconsin.
#     CIDR (Subnet Masks etc) Handling/Notation
#     Alpha Testing, and bug chasing in David's Code.
#   David Bussenschutt <D.Bussenschutt@mailbox.gu.edu.au>, Griffith University, Queensland, Australia.
#     Much re-arrangement, and fixes, including but not limited to:
#     Perl 5 Syntax, and modules
#     File Locking
#     OMAPI Support
#     Remote dhcpdmanager daemon, and client support (functionally similar to omapi, but more fully implemented)
#     Improved Error Handling
#     Use of -w for improved code
#     LDAP group handling, for Permissions.
#     install_script
#   Michael McMaster <m4k3r@users.sourceforge.net>, Griffith University,  Queensland, Australia.
#     DBI/Oracle Support
#     HTML extraction/templating
#     Permissions handling (access controls)
#     Cookie based authentication
#     Interface for editing dhcp options/groups and hosts.
#     Synchronisation between hosts
#     Scalability improvements
#     Various code clean-ups
#     Consistency checker for host information 
#----------------------------------------------------------------------------/
package Variables;
use strict;
use warnings;
#----------------------------------------------------------------------------\\
#
# Exports
#
#----------------------------------------------------------------------------//
use Exporter();
our @ISA = qw(Exporter);

our @EXPORT = qw( );
our @EXPORT_OK =  qw( variables_init $IP $GLOBAL_AUTH_SVR $GLOBAL_AUTH_METHOD
                  $LDAP_BASE $LDAP_USERS $LDAP_URL $GFX $ADMINPATH 
                  $ERROR_LOG $IMPORT_LOG $DDNS_USE_HOST_DECL $MAC $KeyName $Key $HELP_EXTN 
                  $DB_CONNECT $DB_USER $DB_PASS $ORACLE_HOME 
                  $SYNC_FILENAME $SCRIPTS $HTML $RSYNC $SSH $SSH_IDENTITY $PS $GREP $PING
                  $DHCPDMANAGER $DHCPDBINARY $TIMEOFFSET $DEBUG
                  $SSH_USER $SSH_DEST $STATIC_FILE $DHCPDMASTERLOCK $SYNC_LOCK $SYNC_SCRIPT
                  $AUTHFILE $NSUPDATE $DIG $FGREP @ZONES $DHCPOPTIONSFILE @ZONEFILES $COOKIE_TIMEOUT );

#----------------------------------------------------------------------------\\
#
# Imported Modules
#
#----------------------------------------------------------------------------//
# If using mod_perl, the $FindBin::RealBin directory will be set ONCE only
# see startup.pl
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.

#----------------------------------------------------------------------------\\
#
# Constants
#
#----------------------------------------------------------------------------//
# Optional mod_perl support.
# Sets a constant, and ensures the correct exit() function is called.
use constant IS_MODPERL => $ENV{MOD_PERL};
use subs qw(exit);
# Select the correct exit function
*exit = IS_MODPERL ? \&Apache::exit : sub { CORE::exit };

#----------------------------------------------------------------------------\\
#
# Exported Methods
#
#----------------------------------------------------------------------------//

#----------------------------------------------------------------------------\\
# variables_init: Sets up the module for use. Must be executed by executable
# scripts before using  this module.
#
# Parameters: none.
#
# Returns: nothing useful.
#----------------------------------------------------------------------------//
# The current IP of the user. NOT defined unless variables_init is called.
our $IP = undef;
sub variables_init {
    $IP = $ENV{'REMOTE_ADDR'} || '10.0.0.0';
}

#----------------------------------------------------------------------------\\
#
# Global Variables
#
# These are shared within this perl module only.
# These variables are specifically shared between executions if using mod_perl
#----------------------------------------------------------------------------//

# Non-user editable variables
# The MAC address of the machine to be registered.
our $MAC;

# Arrays of DNS zone's for checking of valid input etc.
our @ZONES = (%%%vars{'ZONES'}%%%);

# PATH locations
our $GFX = "/gfx";
our $SCRIPTS = "/scripts";
our $HTML = "%%%paths{'html'}%%%";
our $ADMINPATH = "/cgi-bin/admin";
our $DHCPDMANAGER = "%%%paths{'bin'}%%%";  # ignore this?. Obsolete.?

# Command locations
our $SYNC_SCRIPT = "%%%paths{'cgi-bin'}%%%sync_dhcp_conf.pl";
our $DHCPDBINARY = "%%%vars{'DHCPD_BINARY'}%%%";   # For checking config files with "-t" option
our $NSUPDATE = "%%%vars{'NSUPDATE_BINARY'}%%%";
our $DIG = "%%%vars{'DIG_BINARY'}%%%";
our $FGREP = "%%%vars{'FGREP_BINARY'}%%%";
our $RSYNC = "%%%vars{'RSYNC_BINARY'}%%%";
our $SSH = "%%%vars{'SSH_BINARY'}%%%";
our $PS = "%%%vars{'PS_BINARY'}%%%";
our $GREP = "%%%vars{'GREP_BINARY'}%%%";
our $PING = "%%%vars{'PING_BINARY'}%%%";

# DATA files
# auth.dat defines the privileges each permitted user has to the system
our $AUTHFILE = "%%%paths{'cgi-bin'}%%%auth.dat";
# dhcpd_options.dat defines a list of strings that are syntactically valid dhcpd.conf options, but that don't start with the keyword 'option': 
our $DHCPOPTIONSFILE = "%%%paths{'cgi-bin'}%%%dhcpd_options.dat";

# we need to know where the zone files are so we can 'grep' through them for CNAME data.
our @ZONEFILES = ("%%%paths{'bind-zones'}%%%");

# setting from 0-3 varies the amount of output in the $ERROR_LOG, recommended level is 2
# until system is fully operational and tested, then convert to 0.
our $DEBUG = 2;

# this is THE MAIN/MASTER/PRIMARY/IMPORTANT config file. It should contain NO 'host' records. It and the host records from the database together make up the $SYNC_FILENAME file.
our $STATIC_FILE= '%%%paths{'cgi-bin'}%%%dhcpd.conf.master'; 

# this is where we temporarily write the fully populated config file to until we distribute it to the remote servers.
our $SYNC_FILENAME = "/tmp/dhcpd.conf.master";  

# Log/Lock/Temp files
our $ERROR_LOG = "/tmp/netreg_errors.log"; # errors and assorted log data here.
our $IMPORT_LOG = "/tmp/netreg_import.log"; 
our $DHCPDMASTERLOCK = "/tmp/dhcpd.conf.master.lock"; # a lock file for safety reasons.
our $SYNC_LOCK = "/tmp/netregsync.lock";              # another lock file for safety reasons.

# Authentication info
our $GLOBAL_AUTH_SVR = '%%%vars{'AUTHSERVER'}%%%';
our $GLOBAL_AUTH_METHOD = '%%%vars{'AUTHTYPE'}%%%'; #IMAP,FTP,POP,NONE,LDAP
our $LDAP_BASE = '%%%vars{'LDAPBASE'}%%%';
our $COOKIE_TIMEOUT = 3600; # 1 hour

# only those users in the following LDAP group can use the registration system (set to '' if not used)
our $LDAP_USERS = '%%%vars{'LDAPGROUP'}%%%';

# Use Dynamic DNS ?
our $DDNS_USE_HOST_DECL  = 1;

# Remote keys 
our $KeyName = 'DHCP_UPDATER';
our $Key = '%%%vars{'KEY'}%%%';

# Database configuration:
# (FYI: your database schema should match that in the /db/ directory of the 
#  the instalation files)

# Set this to empty if not using oracle.
our $ORACLE_HOME = "/ora/app/oracle/product/9.2.0";

# the connection string...
our $DB_CONNECT = "%%%vars{'DBTYPE'}%%%%%%vars{'DBNAME'}%%%";
our $DB_USER = '%%%vars{'DBUSERNAME'}%%%';
our $DB_PASS = '%%%vars{'DBPASS'}%%%';

# Host sync settings
our $SSH_USER = "%%%vars{'SSH_USER'}%%%";  # this is the user on the REMOTE servers we will ssh TO.
our $SSH_DEST = "%%%paths{'config'}%%%";   # this is where the dhcpd.conf.* file/s end up on the REMOTE servers.
our $SSH_IDENTITY = "%%%vars{'SSH_IDENTITY'}%%%"; # this is the private key we use for the ssh connection.

# MISC settings
# Set to URL or "" if you do not have an online directory application
# Remember to URL-encode special characters. Leave <USERNAME> where the
# username should be substituted into the URL.
our $LDAP_URL = '%%%vars{'PHONE_URL'}%%%';
#our $LDAP_URL = "https://some.server/phonebook?searchin=<USERNAME>";

# this is the phone number that your users should call when they need IT help
our $HELP_EXTN = "x55555";

# The number of seconds offset the local timezone is from gmt.
# Used in converting DHCP lease times to a more readable format.
our $TIMEOFFSET = 60 * 60 * 10; # +10 hours.

#----------------------------------------------------------------------------/
1;

