#!/usr/bin/perl -w -I%%%paths{'cgi-bin'}%%% -I.
#----------------------------------------------------------------------------\
# NetReg 2.0 (pre-release devel version)
#       DHCPD Registration Frontend
#
# Version 1.3: 
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
# Version 2.0:
# Portions Copyright (C) 1999-2002 Peter Valian, Southwestern University.
# Portions Copyright (C) 2002 David Bussenschutt, Griffith University, Australia.
# Portions Copyright (C) 2002 Robert Lowe,  Lawrence University, Wisconsin.
# Portions Copyright (C) 2002-2003 Michael McMaster, Griffith University, Australia.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Version 1.3: http://www.netreg.org/
# Version 2.0: http://www.sourceforge.net/projects/netreg/
#
# Version 1.3 code contributions by:
#   Peter Valian <valianp@southwestern.edu>,  Southwestern University.
#     Author of Version 1.3rc2 
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#   Michael Grinnell <grinnell@american.edu>, American University
#     LDAP Authentication
#
# Version 2.0 code contributions by:
#   Robert Lowe <robert.h.lowe@lawrence.edu>, Lawrence University, Wisconsin.
#     CIDR (Subnet Masks etc) Handling/Notation
#     Alpha Testing, and bug chasing in David's Code.
#   David Bussenschutt <D.Bussenschutt@mailbox.gu.edu.au>, Griffith University, Queensland, Australia.
#     Much re-arrangement, and fixes, including but not limited to:
#     Perl 5 Syntax, and modules
#     File Locking
#     OMAPI Support
#     Remote dhcpdmanager daemon, and client support (functionally similar to omapi, but more fully implemented)
#     Improved Error Handling
#     Use of -w for improved code
#     LDAP group handling, for Permissions.
#     install_script
#   Michael McMaster <m4k3r@users.sourceforge.net>, Griffith University,  Queensland, Australia.
#     DBI/Oracle Support
#     HTML extraction/templating
#     Permissions handling (access controls)
#     Cookie based authentication
#     Interface for editing dhcp options/groups and hosts.
#     Synchronisation between hosts
#     Scalability improvements
#     Various code clean-ups
#     Consistency checker for host information 
#----------------------------------------------------------------------------/

use warnings;
use strict;
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.
use Database qw( db_connect );
use Variables qw ();
use NetregMisc qw (open_with_lock);
use Subnets qw (add_new_subnet_to_system);

print << "EOF";
Please note!, this script will both modify the dhcpd.conf.master file, AND make the
necessary updates to the database. It will create a new subnet declaration using
the standard layout as defined in the Netreg system.
(Administrators, the declaraion layout is stored in the "add_new_subnet_to_system" routine in the
Subnets.pm file.  You should change the layout in this routine to match your requirements)

EOF

my $dbh = db_connect(1) || die "Could not connect to database.\n";

my ($subnet, $campus, $desc);

my %campuses;
my $sql_campuses = $dbh->prepare ("select distinct campus from campuses");
$sql_campuses->execute();
while (my @campus_row = $sql_campuses->fetchrow_array) {
    $campuses{$campus_row[0]} = 1;
}
$sql_campuses->finish();

while (!$campus) {
    print "Please select the campus to add a subnet to:\n";
    foreach (sort keys %campuses) {
        print "$_\n";
    }
    $campus = <STDIN>;
    chomp($campus);
    $campus = '' unless $campuses{$campus};
}


my $cidrmask = undef; # '24';
while (!$cidrmask) {
    print "Please enter the CIDR subnet mask (ie a number 0-32) for your new subnet (eg 24 or 16):\n";
    $cidrmask = <STDIN>;
    chomp ($cidrmask);
    $cidrmask = '' unless ($cidrmask > 0 && $cidrmask < 32);
}

while (!$subnet) {
    print "Please enter the subnet to add (eg. 132.234.50.0)\n";
    $subnet = <STDIN>;
    chomp ($subnet);
    $subnet = '' unless $subnet =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
}

print "Please enter a description of this subnet. eg. NA subnet 50 (CIT)\n";
$desc = <STDIN>;
chomp ($desc);

my $sql_exist = $dbh->prepare("select * from subnets where cidr_subnet = ? ".
                " and cidr_mask = ? ");
$sql_exist->execute($subnet,$cidrmask);
if ($sql_exist->fetchrow_array()) {
    die "ERROR: Subnet $subnet already exists\n";
}

print "Please enter a list of domains valid for this subnet (comma separated)\n";
print "eg. itc.griffith.edu.au, cit.griffith.edu.au\n";
my $domain = <STDIN>;
chomp($domain);
my @dummy_domains = split(/[\,]/, $domain);
my @domains;
foreach (@dummy_domains) {
    next unless /^\s*[a-z0-9\.\-]+\s*$/i;
    s/^\s*([a-z0-9\.\-]+)\s*$/$1/i;
    push @domains, $_;
}

print "\nPlease restart the webserver when the script finishes. It is safe to run the script without ".
      "stopping the webserver first.\n";

print "About to write details for subnet $subnet [$desc]\n";
print "Campus: $campus\n";
print "Domains: @domains\n";
print "[Y/N]";
my $choice = <STDIN>;
exit unless ($choice =~ /^Y/i);


# add a new subnet now that we have the info.....
add_new_subnet_to_system("NonRoaming",$campus,$cidrmask,$subnet,$desc,@domains);
