#!/usr/bin/perl -w -I/usr/local/apache2/cgi-bin/ -I.
use warnings;
use strict;
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.
use Database qw( db_connect );
use Variables qw ($DHCPDMASTERLOCK $STATIC_FILE $GLOBAL_AUTH_SVR $GLOBAL_AUTH_METHOD $GREP);
use NetregMisc qw (open_with_lock);

my $dbh = db_connect(1) || die "Could not connect to database.\n";

my $subnet;
#my $desc;

my $cidrmask = undef; # '24';
while (!$cidrmask) {
    print "Please enter the CIDR subnet mask (ie a number 0-32) for the subnet you wish to delete (eg 24 or 16):\n";
    $cidrmask = <STDIN>;
    chomp ($cidrmask);
    $cidrmask = '' unless ($cidrmask > 0 && $cidrmask < 32);
}

# convert the CIDR mask into the traditional xxx.xxx.xxx.xxx netmask:
my $netmask = "1" x $cidrmask . "0" x (32 - $cidrmask);
$netmask = join ".", (unpack 'CCCC', pack("B*", $netmask ));

my %subnets;
my $sql_subnets = $dbh->prepare ("select distinct cidr_subnet from subnets where  cidr_mask = ?");
$sql_subnets->execute($cidrmask);
while (my @subnets_row = $sql_subnets->fetchrow_array) {
    $subnets{$subnets_row[0]} = 1;
}
$sql_subnets->finish();

while (!$subnet) {
    print "Please select the subnet to delete:\n";
    foreach (sort keys %subnets) {
        print "$_\n";
    }
    $subnet = <STDIN>;
    chomp($subnet);
    $subnet = '' unless $subnet =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/;
    $subnet = '' unless $subnets{$subnet};
}

my $sql_exist = $dbh->prepare("select * from subnets where cidr_subnet = ? ".
                " and cidr_mask = ? ");
$sql_exist->execute($subnet,$cidrmask);
if (! $sql_exist->fetchrow_array()) {
    die "ERROR: Subnet $subnet with mask $cidrmask does not exist in database\n";
}

my $sql_interface = $dbh->prepare("select mac,hostname,group_id from interface where cidr_subnet = ? and cidr_mask = ? ");
$sql_interface->execute($subnet,$cidrmask);
my $hosts = 0;
while (my @interfaces_row = $sql_interface->fetchrow_array) {
    $hosts++;
    print "$interfaces_row[1]\t$interfaces_row[0]\t$interfaces_row[2]\n";
}
$sql_interface->finish();
if ( $hosts) {
print "I found the above hosts with that subnet information recorded.\n";
print "we can delete the host records entirely (they will be unregistered as a result), or we can remove the subnet/mask info from their record.\n";

my $delete = undef; # '24';
while (!$delete) {
    print "Please enter 'D' to Delete entirely, or 'U' to update their records:\n";
    $delete = <STDIN>;
    chomp ($delete);
    $delete = uc($delete);
    $delete = '' unless ( $delete eq 'D' or $delete eq 'U');
}

    if ( $delete eq 'D') {
      my $sql_delete = $dbh->prepare("delete from interface where cidr_subnet = ? and cidr_mask = ?");
      $sql_delete->execute($subnet,$cidrmask);
      $sql_delete->finish();
    } else {
      my $sql_update = $dbh->prepare("update interface set cidr_subnet = NULL, cidr_mask = NULL where cidr_subnet = ? and cidr_mask = ?");
      $sql_update->execute($subnet,$cidrmask);
      $sql_update->finish();

    }
}

my $sql_domain = $dbh->prepare("delete from subnet_domain where cidr_subnet = ? and cidr_mask = ?");
$sql_domain->execute($subnet,$cidrmask);
$sql_domain->finish();

my $sql_range = $dbh->prepare("delete from ip_range where cidr_subnet = ? and cidr_mask = ? ");
$sql_range->execute($subnet,$cidrmask);
$sql_range->finish();

my $sql_auth = $dbh->prepare("delete from authentication where cidr_subnet = ? and cidr_mask = ?");
$sql_auth->execute($subnet,$cidrmask);
$sql_auth->finish();

# finally delete from the subnets table, now that we've removed all the things that reference this table as a foreign key...
my $sql_subnet = $dbh->prepare("delete from subnets where cidr_subnet = ? and cidr_mask = ? ");
$sql_subnet->execute($subnet, $cidrmask);
$sql_subnet->finish();

print "\nSubnet $subnet with mask $cidrmask is now removed from the database.\n";
print "\n You should now remove all references to this subnet from the dhcpd.conf.master file on the registration server (in cgi-bin).\n";

