#!/usr/bin/perl -w
#----------------------------------------------------------------------------\\
#
# NetReg Test Script for inporting data to oracle.
#
# See included INFO file for licence, author, and contrib information.
#
#----------------------------------------------------------------------------//


use Variables qw( $IMPORT_LOG $DB_CONNECT $DB_USER $DB_PASS $ORACLE_HOME $IP $DHCPDMASTERLOCK);
use Database qw (db_connect);
use NetregMisc qw(open_with_lock make_time append_import_log);
use Authenticate;
use Files_Remote;
use Subnets qw(what_is_my_registered_subnet);
use Admin;
use DBI;

#----------------------------------------------------------------------------/
# Database Handle
my $dbh;

if (!$ARGV[0]) {
    print ("Usage - \nimport.pl filename\n\n");
    print ("filename must contain a list of hosts to import.\n");
    print ("1 host per line, with values comma separated.\n");
    print ("mac,hostname,assetno,owner,domain,status ('R'egistered/dynamic,'S'tatic),expiry date(YYYY-MM-DD),platform,comments,ip ('S),group\n");
    print ("\n eg:\n");
    print ("11:11:22:33:aa:bb,pc000001,000001,s000000,itc.gu.edu.au,R,2020-01-01,Win XP,TEN 1.16 back row,,Roaming:Labs:CIT_LAB\n");
    exit;
}

db_connect();
$ENV{'ORACLE_HOME'} = $ORACLE_HOME;
$dbh = DBI->connect($DB_CONNECT, $DB_USER, $DB_PASS,
                    { AutoCommit => 0,
                      RaiseError => 1}
                   );

my $insert = $dbh->prepare("insert into netreg.interface ".
               " values (?, ?, ?, ?, ?, ?, ?,?,?,?,?,?,?,?)");
my $log = $dbh->prepare ("insert into interface_log ".
               " values (?,?,?,?,?,?,?, ?,'IMPORT',?,?,?)");

#----------------------------------------------------------------------------/
# Batch Sequence Number

my $sql_get_next_batch_id = $dbh->prepare("select batch_id_seq.nextval from dual");
   $sql_get_next_batch_id->execute();

my $sql_sequence = ($sql_get_next_batch_id->fetchrow_array())[0];
   $sql_get_next_batch_id->finish();

open (INPUT, "<$ARGV[0]") || die "Could not open $ARGV[0]\n$!\n";

my $lock_file = open_with_lock("> $DHCPDMASTERLOCK");
if (!$lock_file) {
    die("Could not exclusively open $DHCPDMASTERLOCK lock file.");
}

print "Attempting to import data. If any database errors occur for any host, you will be given the option to rollback ";
print "the import when the script finishes.\n\n";

eval {
my $entrydate = make_time();
    while (<INPUT>) {
        next if /^#/;
        next if /^\s*$/;
        my @row = split(/,/);
        print "Processing host $row[0]\n";
        $row[6] =~ /(....)-(..)-(..)/;
        my $date = "$1$2$3";
        $row[8] = '' unless $row[8];
        $row[9] = '' unless $row[5] eq 'S';
        $row[10] = 'Roaming:Default' unless $row[10];



        my ($cidr_subnet,$cidr_mask) = '';
        if ($row[5] eq 'S') {
            ($cidr_subnet,$cidr_mask) = what_is_my_registered_subnet($row[9]); 
        }
        $insert->execute(uc $row[0],$row[1],$row[2],$row[3],$row[4],$row[5],$date,$row[7],$row[8],$row[9],$cidr_subnet,$cidr_mask,$row[10],$sql_sequence)  || print ("MAC Failed: $row[0]\n");
        $insert->finish();

        $log->execute(uc $row[0],$row[1],$row[2],$row[3],$row[4],$row[5],$date,$entrydate,$row[7],$row[9],$row[8]);
        $log->finish();
    }
};
close INPUT;

print "\n";

if ($@) {
    print "Database errors were detected. Netreg import cancelled. Any hosts imported by this script have been dumped.\n";
    $dbh->rollback();
    $dbh->disconnect();
    exit;
}

print "Batch Sequence Number is $sql_sequence \n";

print "Do you wish to commit these changes ? [y/n]\n";
my $choice = <STDIN>;
chomp ($choice);
if ($choice =~ /^y/i) {
    $dbh->commit();
    print "Changes committed.\n";
    append_import_log("Batch Seq is $sql_sequence for file @ARGV \n");
} else {
    $dbh->rollback();
    print "Changes have been discarded\n";
}
$dbh->disconnect();
exit;
