#!/usr/bin/perl -w -I.
#----------------------------------------------------------------------------\
# NetReg 2.0 (pre-release devel version)
#       DHCPD Registration Frontend
#
# Version 1.3: 
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
# Version 2.0:
# Portions Copyright (C) 1999-2002 Peter Valian, Southwestern University.
# Portions Copyright (C) 2002 David Bussenschutt, Griffith University, Australia.
# Portions Copyright (C) 2002 Robert Lowe,  Lawrence University, Wisconsin.
# Portions Copyright (C) 2002-2003 Michael McMaster, Griffith University, Australia.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Version 1.3: http://www.netreg.org/
# Version 2.0: http://www.sourceforge.net/projects/netreg/
#
# Version 1.3 code contributions by:
#   Peter Valian <valianp@southwestern.edu>,  Southwestern University.
#     Author of Version 1.3rc2 
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#   Michael Grinnell <grinnell@american.edu>, American University
#     LDAP Authentication
#
# Version 2.0 code contributions by:
#   Robert Lowe <robert.h.lowe@lawrence.edu>, Lawrence University, Wisconsin.
#     CIDR (Subnet Masks etc) Handling/Notation
#     Alpha Testing, and bug chasing in David's Code.
#   David Bussenschutt <D.Bussenschutt@mailbox.gu.edu.au>, Griffith University, Queensland, Australia.
#     Much re-arrangement, and fixes, including but not limited to:
#     Perl 5 Syntax, and modules
#     File Locking
#     OMAPI Support
#     Remote dhcpdmanager daemon, and client support (functionally similar to omapi, but more fully implemented)
#     Improved Error Handling
#     Use of -w for improved code
#     LDAP group handling, for Permissions.
#     install_script
#   Michael McMaster <m4k3r@users.sourceforge.net>, Griffith University,  Queensland, Australia.
#     DBI/Oracle Support
#     HTML extraction/templating
#     Permissions handling (access controls)
#     Cookie based authentication
#     Interface for editing dhcp options/groups and hosts.
#     Synchronisation between hosts
#     Scalability improvements
#     Various code clean-ups
#     Consistency checker for host information 
#----------------------------------------------------------------------------/
use strict;
use warnings;

#----------------------------------------------------------------------------\\
#
# Imported Modules
#
#----------------------------------------------------------------------------//
# If using mod_perl, the $FindBin::RealBin directory will be set ONCE only
# see startup.pl
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.

# Netreg modules
use Variables    qw( variables_init $IP $DDNS_USE_HOST_DECL $IP $MAC);

use Subnets      qw( ip_in_subnet ip_in_subnet_urange
	             what_is_my_registered_subnet read_domains);

use Html         qw( html_init register_error print_success admin_error 
                     print_register_page get_input get_cookies print_html_file);

use Files_Remote qw( read_conf_new_remote append_conf_remote 
                     get_host_info_remote_all );

use Database     qw( db_connect db_new_host db_already_registered
                     db_next_serial db_unique_host);

use Authenticate qw( auth_init is_valid_cookie logon user priv logoff);

use NetregMisc   qw( make_time ); 

#----------------------------------------------------------------------------\\
#
# Constants
#
#----------------------------------------------------------------------------//
# Optional mod_perl support.
# Sets a constant, and ensures the correct exit() function is called.
use constant IS_MODPERL => $ENV{MOD_PERL};
use subs qw(exit);
# Select the correct exit function
*exit = IS_MODPERL ? \&Apache::exit : sub { CORE::exit };

#----------------------------------------------------------------------------\\
#
# Main Function
#
#----------------------------------------------------------------------------//

variables_init();
html_init();
auth_init();

if (!db_connect()) {
    admin_error("Could not connect to Database","Database");
    print_html_file("register_footer.html");
}

# Note: %form will NOT be shared with mod_perl, even though it's at
# the outer scope (mod_perl will wrap this entire script, 'causing it
# to be in an inner scope)
my %form = get_input();

do_logon();

if ($form{'action'} && $form{'action'} eq 'Logoff') {
    logoff(user());
    my $image = "background-image: url(\"/gfx/logo_6.jpg\");\n".
                "background-repeat: no-repeat;\n".
                "background-position: center center;\n";
    print_html_file('register_header.html', "dummy;", $image);
    print_html_file("register_logon.html"); 
    print_html_file("register_footer.html");
}

print_html_file("register_header.html"); #must be after logon for cookies.

# begin the work!
if (!$form{'registering'}) {
    my $domains;
    $domains = read_domains($IP);
    print_register_page(priv('custom_hostname'), @$domains); # Will not return.
}

# If print_register_page isn't run, then we want to process a request.
check_input(); 
get_host_info();
append_host_entry();
print_success();
exit;

#----------------------------------------------------------------------------\\
#
# Methods
#
#----------------------------------------------------------------------------//

#----------------------------------------------------------------------------\\
# do_logon: attempts to authenticate a user with the system, using either
# a provided username/password or a session cookie.
#
# Parameters: None.
#
# Returns: Nothing useful. Will display a logon page on failure and will exit.
#----------------------------------------------------------------------------//
sub do_logon {
  my %cookies = get_cookies();
  my %form = get_input();

  # Check the cookie.
  my $user;
  if ($cookies{'netreg-session'}) {
    $user  = is_valid_cookie($cookies{'netreg-session'});
  }

  # Cookie failed. Check for manual logon.
  if (!$user) {
      if ($form{'user'} && $form{'pass'}) {
          # Check the given username/password.
          # will not return if it failed.
          logon ($form{'user'}, $form{'pass'});

      } else {
          my $image = "background-image: url(\"/gfx/logo_6.jpg\");\n".
                      "background-repeat: no-repeat;\n".
                      "background-position: center center;\n";
          print_html_file('register_header.html', "dummy;", $image);
          print_html_file("register_logon.html");
          print_html_file("register_footer.html");
      }
  }

  if (!priv('logon_register')) {
      print_html_file("register_header.html");
      admin_error("Insufficient Privileges", "Privilege");
      print_html_file("register_footer.html");
  }
}

#----------------------------------------------------------------------------\\
# check_input: Ensure the user has provided valid input.
#
# Parameters: None.
#
# Returns: Nothing. Will display an error message and exit on error.
#----------------------------------------------------------------------------//
sub check_input {
    my %form = get_input();

    # Check assest numbers
    if ($form{'asset'} !~ /^\d\d\d\d\d\d$/) {
        register_error("Asset Number must be 6 digits exactly. ".
            "Please prefix it with Zero's if it's a shorter number.");
    }

    # Check staff/student numbers
    if ($form{'owner'} !~ /^s\d+$/i && !priv('relaxed_s_number')) {
        register_error("Owner must be 's' followed by numbers.");
    }
    $form{'owner'} = lc($form{'owner'});

    # Check expiry date
    if ($form{'expiry'} && $form{'expiry'} !~ /^\d\d\/\d\d\/\d\d\d\d$/) {
        register_error("Date must be in the format of DD/MM/YYYY.");
    }
   
    # TODO this code may be optimised using an SQL query.
    # Check for a valid domain 
    my @domain_array = @{read_domains()};
    my $found_domain = undef;
    foreach my $domain (@domain_array) {
        if ($domain eq $form{'domain'}) {
            $found_domain = "1";
            last;
        }
    }
    if (!$found_domain) {
        register_error("Please select a valid domain from the drop-down list.");
    }
    
    # Check hostnames
    if ($form{'hostname'} && $form{'hostname'} !~/^[a-z0-9\-]/i) {
        register_error("Host name may only contain alphanumeric characters ".
                       "and '-'");
    }
    if ($form{'hostname'} && !priv('custom_hostname')) {
        register_error("Only superusers of the system may specify hostnames.");
    }  
    if ($form{'hostname'} && !db_unique_host($form{'hostname'}, $form{'domain'})) {
        register_error("Hostname is not unique.");
    }
}

#----------------------------------------------------------------------------\\
# check_subnet: Ensure that the user attempting to register is on a known
# subnet.
#
# Parameters: none.
#
# Returns: The subnet of the user, or will exit on error.
#----------------------------------------------------------------------------//
sub check_subnet {
    my ($registered_subnet,$mask) = what_is_my_registered_subnet($IP);
    if (!$registered_subnet) {
        register_error("Permission Denied<P>You are not allowed to register ".
                       "based on your IP address (unknown subnet).<br>$IP");
    }
    return ($registered_subnet, $mask);
}

#----------------------------------------------------------------------------\\
# get_host_info: Attempt to find a MAC address from a DHCP lease for the 
# machine attempting to register.
#
# Parameters: None.
#
# Returns: Nothing, will exit on error.
#----------------------------------------------------------------------------//
sub get_host_info {
    $MAC =  get_host_info_remote_all($IP);

    if (! defined $MAC ) {
        register_error ("Could not find MAC address for $IP based on current ".
                        "network settings. Remote servers may be down.");
    }

    if (!$MAC) { # this is a truth test (ie is string more than zero length)
        register_error("No Lease Found for your IP of: $IP. ".
                       "Is this PC configured to DHCP?");
    }
}


#----------------------------------------------------------------------------\\
# append_host_entry: Add a machine's details to the remote DHDPD server configs.
#
# Parameters: None.
#
# Returns: Nothing.
#----------------------------------------------------------------------------//
sub append_host_entry {
    my %form = get_input();
    my ($REG_subnet,$mask) = check_subnet();

    # Get the time and put it in the format we want
    my $timestamp = make_time();

    # Check for duplicates
    if (db_already_registered($MAC)) {
        register_error("Hardware Already Registered. ".
                       "Please Reboot Your Computer.");
    }

    my $host;
    if ($form{'hostname'}) {
        $host = $form{'hostname'};
    } else {
        my $serial;
        $serial = db_next_serial($form{'asset'}, $MAC);

        $host = "pc".$form{'asset'};
        $host .= "-".$serial if $serial;
    }

    my $expiry = $form{'expiry'};
    if (!$expiry) {
        $expiry = make_time();
        $expiry =~ /^(\d\d\d\d)(\d\d)(\d\d).*$/;
        my ($day,$month,$year) = ($3,$2,$1+5);
        $expiry = "$year$month$day";
    } else {
        $expiry =~ /(\d+).(\d+).(\d+)/;
        my $year = $3 > 2030 ? 2030 : $3;
        $expiry = "$year$2$1";
    }

    my $domain = $form{'domain'};
    my $owner = $form{'owner'}; 
    my $asset = $form{'asset'}; 

    my $browser;
    if ($form{'mac'}) {
        $browser = 'Unknown';
        $IP = '0.0.0.0';
    } else {
        $browser = $ENV{'HTTP_USER_AGENT'};
    }
    
    # Insert entry into the database
    # This call will also log the attempt to the database.
    if (!db_new_host($MAC, $host, $asset, $owner, $domain, 'R', $expiry,
                     user(), $browser, $IP, $form{'comment'})) {

        register_error("Creation of host in database failed. ".
                       "Check that all data entered is valid.");
    }
    
    # Push the registration out to the DHCP servers on the subnet
    # from where the registration originated. Registration to
    # other servers will be delayed up to 15 minutes.

    # Add DHCP option to use derived name in DDNS updates, if desired
    my @append = ("host $host { hardware ethernet $MAC \; }\n");
    if ($DDNS_USE_HOST_DECL) {
        # DDNS
        $append[1] = 
               "subclass \"match-hardware\" 1:$MAC { ".
               "ddns-hostname \"$host\"; option domain-name \"$domain\"; }\n";
    }

    my $dbh = db_connect();
    if (!$dbh) {
        admin_error("Failed to get server list from database.");
    } else {
        my $sql_server = $dbh->prepare("select c.server_ip from campuses c, subnets s ".
                "where s.campus = c.campus and cidr_subnet = ? and cidr_mask = ?");
            
        $sql_server->execute($REG_subnet, $mask); 
        while (my @row = $sql_server->fetchrow_array) {
            my $err = append_conf_remote($row[0],@append);
            admin_error("Failed to save registration to $row[0].") 
                unless defined $err;
        }
        $sql_server->finish();
    }
}
