#!/usr/bin/perl -w -I/opt/apache2/cgi-bin/ -I.
#
#----------------------------------------------------------------------------\\
#
# NetReg Periodic DHCPD Synchronisation Script (from Database to config file/s)
#
# See included INFO file for licence, author, and contrib information.
#
#----------------------------------------------------------------------------//

use warnings;
use strict;
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.

use Variables qw( $SYNC_LOCK $DDNS_USE_HOST_DECL $SYNC_FILENAME $SSH_USER $SSH_DEST $STATIC_FILE $DHCPDMASTERLOCK $RSYNC $SSH $SSH_IDENTITY $GREP $PS $DEBUG);
use Database qw( db_connect db_disconnect db_get_unregip);
use NetregMisc qw (append_error_log open_with_lock make_time);
use StaticHosts qw (parse_conf);
use POSIX ":sys_wait_h";

#----------------------------------------------------------------------------\
# Main Function
#----------------------------------------------------------------------------\
# check to see if more than 1 other sync process has started.
my $ps = `$PS | $GREP -v grep | $GREP 'perl' | $GREP 'sync'`;
my @lines = split(/\n/, $ps);
if ($#lines > 1) {
    append_error_log("sync_dhcp_conf.pl: Too many simultaneous sync's\n");
    exit(1);
}

my $dbh = db_connect(1);
if (!$dbh) {
    append_error_log ("sync_dhcp_conf.pl: Could not connect to database.\n");
    exit(1);
}

my $lock = open_with_lock(">$SYNC_LOCK");
if (!$lock) {
    append_error_log("sync_dhcp_conf.pl: Could not lock $SYNC_LOCK. Aborting sync\n");
    exit(2);
}
my $lock2 = open_with_lock(">$DHCPDMASTERLOCK");
if (!$lock2) {
    append_error_log("sync_dhcp_conf.pl: Could not lock file $DHCPDMASTERLOCK");
    exit(1);
}

my $sql_all_servers = $dbh->prepare("select distinct server_ip from campuses");
$sql_all_servers->execute();
my @servers;
while (my @row = $sql_all_servers->fetchrow_array) {
    push @servers, $row[0];
}
$sql_all_servers->finish();

append_error_log("sync_dhcp_conf.pl: About to write populated config file\n") if $DEBUG>=3;
write_file();
$dbh->disconnect();
append_error_log("sync_dhcp_conf.pl: checking config file size..\n") if $DEBUG>=3;
check_config_size();
append_error_log("sync_dhcp_conf.pl: distributing config file to remote servers..\n") if $DEBUG>=3;
push_new_config(@servers);
close $lock;
close $lock2;
exit(0);

#----------------------------------------------------------------------------/


#############################################################################\
## SUBROUTINES BEGIN
#############################################################################/

#----------------------------------------------------------------------------\
# Write all valid hosts (as stored in the database) to a new DHCP conf file.
#----------------------------------------------------------------------------\
sub write_file {

    my $newconf = open_with_lock (">$SYNC_FILENAME.check");
    if (!$newconf) {
        append_error_log("sync_dhcp_conf.pl: Error: can't lock $SYNC_FILENAME file: $!\n");
        exit(1);
    }

    my $template = parse_conf();
    if (!$template) {
        append_error_log("sync_dhcp_conf.pl: Could not parse dhcpd.conf.master file.\n");
        exit(1);
    }
    $template->write($newconf,0,'',1);
    close $newconf;
}
#----------------------------------------------------------------------------\
# Checking file sizes and exsistance.
#----------------------------------------------------------------------------\
sub check_config_size {

  if ( ! -f "$SYNC_FILENAME.check" || ! -s "$SYNC_FILENAME.check" ) {   
	# throw error here since XXX.check file doesn't exist 
        append_error_log("sync_dhcp_conf.pl: $SYNC_FILENAME.check not valid file or zero size.\n");
        print "$SYNC_FILENAME.check not valid file or zero size.\n" if $DEBUG>=1;
 	exit(1);
  }
  if ( ! -f $SYNC_FILENAME || ! -s $SYNC_FILENAME ) {   
	# throw error here since XXX  file doesn't exist 
        append_error_log("sync_dhcp_conf.pl: $SYNC_FILENAME not valid file or zero size. \nIf this is the first time you have run this script, please rename $SYNC_FILENAME.check to $SYNC_FILENAME and re-run it.\n");
        print "$SYNC_FILENAME not valid file or zero size. \nIf this is the first time you have run this script, please rename $SYNC_FILENAME.check to $SYNC_FILENAME and re-run it.\n" if $DEBUG>=1;
 	exit(1);
  }

  my $t = (-s "$SYNC_FILENAME.check");
  my $t2 = (-s "$SYNC_FILENAME");
 
  my $t3 = (($t/$t2) > .9 );

  if (! $t3 > .9 ) {

   # throw error. 
   append_error_log("sync_dhcp_conf.pl: $SYNC_FILENAME.check is MUCH smaller than $SYNC_FILENAME. Not valid file or zero size.\n") if $DEBUG>=1;
   print "$SYNC_FILENAME.check is MUCH smaller than $SYNC_FILENAME. Not valid file or zero size.\n" if $DEBUG>=1;
   exit(1);

  }  
   system ('/bin/cp',$SYNC_FILENAME.'.check',$SYNC_FILENAME);
   chmod 0664, $SYNC_FILENAME.'.check',$SYNC_FILENAME;
   system ('/bin/chgrp','netreg',$SYNC_FILENAME.'.check',$SYNC_FILENAME);

   append_error_log("sync_dhcp_conf.pl: $SYNC_FILENAME is within size limits.\n") if $DEBUG>=1;
   print "$SYNC_FILENAME is within size limits.\n" if $DEBUG>=1;
	
}

#----------------------------------------------------------------------------\
# Uses sync to send config file to all nodes
#----------------------------------------------------------------------------\
sub push_new_config {
    my @servers = @_;
    print `cp $SYNC_FILENAME $SYNC_FILENAME.new`;

    print "distributing new config file to:\n" if $DEBUG >=1;

    my @pids = ();
    foreach my $server (@servers) {
        my $pid = fork();
        push @pids, $pid if $pid;
        next if $pid;
        my $command = "$RSYNC -ze'$SSH -a -c blowfish -x -1 -i $SSH_IDENTITY' $SYNC_FILENAME.new $SSH_USER@".
                      "$server:$SSH_DEST/";
        my $returnval = system(`$command`);
        if ($DEBUG>=1 && !$returnval) {
          append_error_log("sync_dhcp_conf.pl: $RSYNC failed to $server!!") ;
        }
        print "\t$server\n" if $DEBUG>=1;
        exit(1);
      
    }

    my $kid;
    do { 
	$kid = waitpid(-1,&WNOHANG);
    } until $kid == -1;  

    `rm $SYNC_FILENAME.check`; # leave the original $SYNC_FILENAME for comparing with later!
    `rm $SYNC_FILENAME.new`;
}
