#!/usr/bin/perl -w
#
#
# client module to the dhcpdmanager daemon 
#
# By David Bussenschutt September 2002
#
package DhcpdManager;
use FindBin;
use lib "$FindBin::RealBin"; # locate library path for below modules.
use strict;
use warnings;

# Possible mod_perl support
use constant IS_MODPERL => $ENV{MOD_PERL};
use subs qw(exit);
# Select the correct exit function
*exit = IS_MODPERL ? \&Apache::exit : sub { CORE::exit };

use Exporter();
our @ISA = qw (Exporter);
our @EXPORT = ( );
our @EXPORT_OK = qw ();
our $VERSION = 1.3.2;

use Socket;
use Digest::HMAC_MD5 qw(hmac_md5 hmac_md5_hex );
use MIME::Base64;
use Symbol;

my $DEBUG=0;

sub new {
# Creates a new object,
# and also creates the objects socket
# and also connects that socket, to the 'server' that was defined
# by either the default settings, or as passed to the 'new'

	my $invocant = shift;
	my $class =  ref($invocant) || $invocant; # object or class name

	my $self = { 
        # default settings.
	port => '7912',
	randlen => 40,
	base64 => 1,
	@_,   # user provided settings may override defaults.
	 };
        # $self->{AUTHOK}=0; # not required as undef is equivalent
        ${$self->{handle}} = Symbol::gensym(); # get a reference to a typeglob

	socket(${$self->{handle}},PF_INET,SOCK_STREAM,getprotobyname('tcp')); # make socket
        setsockopt(${$self->{handle}}, SOL_SOCKET, SO_KEEPALIVE, 1);
	my $i_addr = inet_aton($self->{server}) or return undef ; #die "error in inet_aton: $!\n";
	my $my_addr = sockaddr_in($self->{port},$i_addr); # make address
	connect (${$self->{handle}},$my_addr) or return undef ; #die "couldnt connect to ".$self->{server},": ",$self->{port},": $!\n";
	select((select(${$self->{handle}}), $|=1)[0]);

	return bless $self, $class; # return the ojbect
}

# Make sure the other end is still up and running...
sub ping {
    my $self = shift;
    
    $self->printb64('ping');
    my $data = $self->readb64();
    if (!$data) {
        return undef;
    } elsif ($data eq 'OK') {
        return "1";
    } else {
        return undef;
    }    
}
    

sub auth {
# perform the bi-directional hand-shake authentication that the server
# requires before it will allow you to query it for anything else.

  my $self = shift;
  my $order = 0;
  my $randdigest;
  print STDERR "auth() started\n" if $DEBUG>1;
  while (1) {  #infinite loop is only ever exited using exit statements to exit
      my $data = $self->readb64();
      print STDERR "auth() readb64 done\n" if $DEBUG>1;
      return undef unless $data; # we are at EOF (ie the socket was closed from the other side)
      $data =~ s/\r//go;
      $data =~ s/\n//go;
      print STDERR "DATA:",$data,"\n" if $DEBUG>1;
      # SWITCH on the info from the data
      for ($data) {  # loop structure is actioned once, and used as a SWITCH statement
        study $_;
        # CONNECTION and PEER-PEER VALIDATION
        /^VERIFY\s+(.*)/o        and do { return undef unless $order == 0;
                                        $order ++;
                                        $self->verify_random($1);
                                        last; };
        /^OK$/o                  and do { return undef unless $order == 1;
                                        $order ++;
                                        $randdigest = $self->peer_connect();
                                        last; };
        /^HANDSHAKE\s+(.*)/o     and do { return undef unless $order == 2;
                                        $order ++;
                                        return undef unless $self->handshake_random($randdigest,$1);
                                        last; };
        /^CONNECT$/o             and do { return undef unless $order == 3;
                                        $order ++;
                                        $self->peer_ok(); };
      }
      if ($order == 4 ) { 
			$self->{AUTHOK} = 1; 
			print STDERR "auth() done\n" if $DEBUG >1;
			return 1 ;
			} ;
  }
}

#  function for accessing object content.
# eg: my $retvalue = obj->get('port'); 
sub get {
	my $self = shift;
	my $info = shift;
	return $self->{$info};
}

sub DESTROY {
	my $self = shift;
	close ${$self->{handle}};  # close socket
}

#---------------------------------
# Base64 handling of strings... both use $base64 global to decide what to do.
# 
sub printb64 {
# print the data to the socket in either base64 encoding, or normally
  my $self = shift;
  my $sock = ${$self->{handle}};
  my @strings = @_;

  if ($self->{base64}) {   # to base64 ...
    foreach (@strings) { 
       print "send: ",encode_base64($_,''),"\n" if $DEBUG;
	print $sock encode_base64($_,''),"\n"; 
    }
  } else {                 # ...or not to base64...
    foreach (@strings) {  
	s/\\n/n/g; s/\n/\\n/go; 
	print "send: ",$_,"\n" if $DEBUG; 
	print $sock $_,"\n"; 
    };
  }
}

sub readb64 {
# read a string, which might be either base64 encoded, or not.
  my $self = shift;
  my $sock = ${$self->{handle}};   # here's the tricky socket handling bit..!
  my $string = <$sock>; # grab a line from the socket, block if it's not avail
  return undef unless defined $string; # we are at EOF (ie the socket was closed from the other side)
  print "recv: $string\n" if $DEBUG;
  my $s = $string;
  if ($self->{base64}) { 
    chomp ($s);
    $s = decode_base64($s);     
  } else { 
    chomp ($s);
    $s =~ s/\\n/\n/go;
  } 
  return $s;
}

# ->send is equivalent to ->printb64
sub send {
  my $self = shift;
  return $self->printb64(@_);
}

# ->recv is equivalent to ->readb64
sub recv {
  my $self = shift;
  return $self->readb64(@_);
}

#---------------------------------
# continue to read data from the given socket/handle
# until a 'END' line is recieved (ie /^END$/)
sub read_data {
my $self = shift;
#my $sock = ${$self->{handle}};
my @data = ();
my $t;
while ( my $data = $self->readb64()) { ;
      return unless $data; # we are at EOF (ie the socket was closed from the other side)
      # HACK: reverse the following two lines to leave/remove the 'END' lines in/from the output
      return @data if $data =~ /^END$/o; 
      push @data,$data;
}
} 

#---------------------------------
# These subroutines are used to perform peer-peer validation between two hosts using a shared HMAC-MD5 key
#
sub peer_connect {
my $self = shift;
# this is SERVER and CLIENT CODE:
#as SERVER peer has just sent 'CONNECT', as CLIENT peer has just sent OK, so we send a VERIFY
  my $randstring = join('', map( chr(33+rand 93), 1 .. $self->{randlen})) ; # a random string of $randlen base64 valid chars
  $self->printb64("VERIFY $randstring");
  # calculate the HMAC_MD5 digest by using the known omapi secret, and return this info for verification elsewhere.
  my $hmac = Digest::HMAC_MD5->new($self->{omapi_secret});
  $hmac->add($randstring);
  my $digest = $hmac->b64digest;
  return $digest;
}

sub handshake_random {
my $self = shift;
# this is SERVER and CLIENT code:
# peer sent a HANDSHAKE so we verify, and send OK or abort.
  my $serverdigest = shift;
  my $clientdigest = shift;
  if ($serverdigest eq $clientdigest) {
    $self->printb64("OK");
    return 1;
  } else {
    $self->printb64("ERR $serverdigest NOT-EQ $clientdigest");
    #$self->DESTROY; # force kill of the TCPIP connection
    undef $self; # force kill of the TCPIP connection
    return undef;
  }
}

sub verify_random {
my $self = shift;
# this is SERVER and CLIENT code:
# peer sent a VERIFY, so we send a HANDSHAKE
  my ($randstring) = @_;
  # calculate the HMAC_MD5 digest by using the known omapi secret, and send this to the socket
  my $hmac = Digest::HMAC_MD5->new($self->{omapi_secret});
  $hmac->add($randstring);
  my $digest = $hmac->b64digest;
  $self->printb64("HANDSHAKE $digest");
}

sub peer_ok {
# this is CLIENT CODE: 
#peer sent 'CONNECT', so we are now free to send a command
my $self = shift;

return;
}


1;
#---------------------------------
